searchData={"items":[{"type":"task","doc":"Generate and configure a custom Cinder filter based on built-in filters.\n\nThis task creates a new custom filter module that delegates to a built-in filter\nas a starting point, allowing you to customize specific behaviors while keeping\nthe rest of the implementation. It automatically registers the filter in your\napplication configuration.","title":"mix cinder.gen.filter","ref":"Mix.Tasks.Cinder.Gen.Filter.html"},{"type":"task","doc":"```bash\nmix cinder.gen.filter MyApp.Filters.CustomText custom_text --template=text\n\n```","title":"Example - mix cinder.gen.filter","ref":"Mix.Tasks.Cinder.Gen.Filter.html#module-example"},{"type":"task","doc":"* `module` - The module name for the filter (e.g., MyApp.Filters.CustomText)\n* `type` - The filter type identifier (e.g., custom_text)","title":"Arguments - mix cinder.gen.filter","ref":"Mix.Tasks.Cinder.Gen.Filter.html#module-arguments"},{"type":"task","doc":"* `--template` or `-t` - Base filter to copy from: text, select, multi_select, boolean, date_range, number_range\n* `--no-tests` - Skip generating test file\n* `--no-config` - Skip automatic configuration registration\n* `--no-setup` - Skip ensuring Cinder.setup() is called in application","title":"Options - mix cinder.gen.filter","ref":"Mix.Tasks.Cinder.Gen.Filter.html#module-options"},{"type":"task","doc":"* `text` - Based on Cinder.Filters.Text (text input with operators)\n* `select` - Based on Cinder.Filters.Select (dropdown selection)\n* `multi_select` - Based on Cinder.Filters.MultiSelect (multiple selection)\n* `boolean` - Based on Cinder.Filters.Boolean (true/false/any selection)\n* `date_range` - Based on Cinder.Filters.DateRange (from/to date picker)\n* `number_range` - Based on Cinder.Filters.NumberRange (from/to number input)","title":"Available Templates - mix cinder.gen.filter","ref":"Mix.Tasks.Cinder.Gen.Filter.html#module-available-templates"},{"type":"task","doc":"1. Creates a custom filter module that delegates to the chosen built-in filter\n2. Automatically updates the filter type to your custom type\n3. Generates comprehensive test file (unless --no-tests)\n4. Automatically adds the filter to your :cinder configuration\n5. Provides clear examples of how to customize the behavior","title":"What it does - mix cinder.gen.filter","ref":"Mix.Tasks.Cinder.Gen.Filter.html#module-what-it-does"},{"type":"task","doc":"The generated filter starts by delegating all behavior to the base filter but with\nyour custom type. You can then override any callback function to customize:\n\n* `render/4` - Customize the HTML rendering\n* `process/2` - Customize input processing and validation\n* `validate/1` - Customize filter validation logic\n* `build_query/3` - Customize query building\n* `default_options/0` - Customize default options\n* `empty?/1` - Customize empty value detection\n\nThe generated filter will be immediately ready to use in your tables.","title":"Customization - mix cinder.gen.filter","ref":"Mix.Tasks.Cinder.Gen.Filter.html#module-customization"},{"type":"task","doc":"Installs Cinder and configures Tailwind CSS to include Cinder's styles.\n\nThis installer ensures that all Tailwind CSS classes used by Cinder are\nincluded in your application's generated stylesheets by updating your\nTailwind configuration.","title":"mix cinder.install","ref":"Mix.Tasks.Cinder.Install.html"},{"type":"task","doc":"For new installations, use Igniter:\n\n```bash\nmix igniter.install cinder\n```\n\nThis will automatically add Cinder to your dependencies and run this installer.","title":"Recommended Installation - mix cinder.install","ref":"Mix.Tasks.Cinder.Install.html#module-recommended-installation"},{"type":"task","doc":"If you've already added Cinder to your dependencies manually:\n\n```bash\nmix cinder.install\n```","title":"Manual Installation - mix cinder.install","ref":"Mix.Tasks.Cinder.Install.html#module-manual-installation"},{"type":"task","doc":"1. Adds Cinder's files to your Tailwind configuration content paths\n2. Provides setup instructions for using Cinder in your application\n3. Shows example usage to get you started quickly","title":"What it does - mix cinder.install","ref":"Mix.Tasks.Cinder.Install.html#module-what-it-does"},{"type":"task","doc":"The installer will attempt to automatically update your Tailwind configuration:\n\n- **Tailwind v3 and below**: Updates `tailwind.config.js` content array\n- **Tailwind v4**: Adds `@source` directive to your CSS file\n\nIf automatic configuration fails, manual setup instructions will be provided.","title":"Tailwind Configuration - mix cinder.install","ref":"Mix.Tasks.Cinder.Install.html#module-tailwind-configuration"},{"type":"task","doc":"After installation:\n\n1. Add Cinder.setup() to your Application.start/2 function (if using custom filters)\n2. Start using Cinder tables in your LiveView templates\n3. Explore the documentation for advanced features","title":"Next Steps - mix cinder.install","ref":"Mix.Tasks.Cinder.Install.html#module-next-steps"},{"type":"module","doc":"Cinder is a powerful, intelligent data table component for Phoenix LiveView applications with seamless Ash Framework integration.","title":"Cinder","ref":"Cinder.html"},{"type":"module","doc":"The simplest table requires only a resource (or query) and actor:\n\n    # Using resource parameter\n     \n      <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n      <:col :let={user} field=\"email\" filter>{user.email}</:col>\n      <:col :let={user} field=\"created_at\" sort>{user.created_at}</:col>\n     \n\n    # Using query parameter\n     \n      <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n      <:col :let={user} field=\"email\" filter>{user.email}</:col>\n      <:col :let={user} field=\"created_at\" sort>{user.created_at}</:col>\n     \n\n    # Advanced query usage\n      Ash.Query.filter(active: true)} actor={@current_user}>\n      <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n      <:col :let={user} field=\"email\" filter>{user.email}</:col>\n      <:col :let={user} field=\"created_at\" sort>{user.created_at}</:col>","title":"Quick Start - Cinder","ref":"Cinder.html#module-quick-start"},{"type":"module","doc":"Cinder supports custom filter types through a simple configuration-based approach:\n\n    # 1. Configure your filters in config.exs\n    config :cinder, :filters, [\n      slider: MyApp.Filters.Slider,\n      color_picker: MyApp.Filters.ColorPicker\n    ]\n\n    # 2. Set up Cinder in your application.ex\n    def start(_type, _args) do\n      Cinder.setup()  # Automatically registers all configured filters\n      # ... rest of application startup\n    end\n\n    # 3. Create custom filters with minimal boilerplate\n    defmodule MyApp.Filters.Slider do\n      use Cinder.Filter  # Includes everything you need\n\n      @impl true\n      def render(column, current_value, theme, assigns) do\n        # Your filter implementation with HEEx templates\n      end\n\n      @impl true\n      def process(raw_value, column) do\n        # Transform form input to filter value\n      end\n\n      @impl true\n      def validate(filter_value), do: # Validate filter value\n\n      @impl true\n      def default_options, do: [min: 0, max: 100, step: 1]\n\n      @impl true\n      def empty?(value), do: # Check if filter is empty\n    end\n\n    # 4. Use in your tables\n    <:col field=\"price\" filter={:slider} filter_options={[min: 0, max: 1000]}>\n      ${product.price}\n    </:col>\n\nThat's it! No manual registration, no complex setup - just configure and use.","title":"Custom Filters - Cinder","ref":"Cinder.html#module-custom-filters"},{"type":"module","doc":"- **Intelligent Defaults**: Automatic filter type detection from Ash resource attributes\n- **Minimal Configuration**: 70% fewer attributes required compared to traditional table components\n- **URL State Management**: Filters, pagination, and sorting synchronized with browser URL\n- **Relationship Support**: Dot notation for related fields (e.g., `user.department.name`)\n- **Custom Filters**: Pluggable filter system with configuration-based registration\n- **Flexible Theming**: Built-in presets and full customization\n- **Ash Integration**: Native support for Ash Framework resources and authorization","title":"Key Features - Cinder","ref":"Cinder.html#module-key-features"},{"type":"module","doc":"- `Cinder.Table` - The main table component\n- `Cinder.Table.UrlSync` - URL state management helper\n- `Cinder.Filter` - Base behavior for custom filters\n- `Cinder.setup/0` - One-time setup for custom filters\n\nFor comprehensive examples and documentation, see the [README](readme.html) and [Examples](examples.html).","title":"Main Components - Cinder","ref":"Cinder.html#module-main-components"},{"type":"function","doc":"Sets up Cinder with configured custom filters.\n\nThis is the recommended way to use custom filters in Cinder. Call this\nfunction once during application startup to automatically register all\ncustom filters defined in your configuration.","title":"Cinder.setup/0","ref":"Cinder.html#setup/0"},{"type":"function","doc":"Define your custom filters in your application configuration:\n\n    config :cinder, :filters, [\n      slider: MyApp.Filters.Slider,\n      color_picker: MyApp.Filters.ColorPicker\n    ]","title":"Configuration - Cinder.setup/0","ref":"Cinder.html#setup/0-configuration"},{"type":"function","doc":"Call `setup/0` once in your application startup:\n\n    # In your application.ex\n    def start(_type, _args) do\n      Cinder.setup()  # Registers all configured filters automatically\n\n      children = [\n        # ... your application's children\n      ]\n\n      opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n      Supervisor.start_link(children, opts)\n    end","title":"Usage - Cinder.setup/0","ref":"Cinder.html#setup/0-usage"},{"type":"function","doc":"- Registers all custom filters from configuration\n- Validates that all filter modules implement the required behavior\n- Provides helpful logging for successful registrations and errors\n- Gracefully handles configuration issues without breaking startup","title":"What it does - Cinder.setup/0","ref":"Cinder.html#setup/0-what-it-does"},{"type":"function","doc":"Always returns `:ok`. Any registration issues are logged as warnings\nbut don't prevent application startup.","title":"Returns - Cinder.setup/0","ref":"Cinder.html#setup/0-returns"},{"type":"function","doc":"# Successful setup with filters\n    Cinder.setup()\n    # => :ok\n    # Logs: \"Cinder: Registered 3 custom filters: slider, color_picker, date_picker\"\n\n    # Setup with configuration errors\n    Cinder.setup()\n    # => :ok\n    # Logs: \"Cinder: Some custom filters failed to register: ...\"","title":"Examples - Cinder.setup/0","ref":"Cinder.html#setup/0-examples"},{"type":"module","doc":"Simplified Cinder table component with intelligent defaults.\n\nThis is the new, simplified API for Cinder tables that leverages automatic\ntype inference and smart defaults while providing a clean, Phoenix LiveView-like interface.","title":"Cinder.Table","ref":"Cinder.Table.html"},{"type":"module","doc":"","title":"Basic Usage - Cinder.Table","ref":"Cinder.Table.html#module-basic-usage"},{"type":"module","doc":"<:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"email\" filter>Email</:col>\n      <:col field=\"created_at\" sort>Created</:col>","title":"With Resource Parameter (Simple) - Cinder.Table","ref":"Cinder.Table.html#module-with-resource-parameter-simple"},{"type":"module","doc":"<!-- Using resource as query -->\n     \n      <:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"email\" filter>Email</:col>\n      <:col field=\"created_at\" sort>Created</:col>\n     \n\n    <!-- Pre-configured query with custom read action -->\n     \n      <:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"email\" filter>Email</:col>\n      <:col field=\"created_at\" sort>Created</:col>\n     \n\n    <!-- Query with base filters -->\n      Ash.Query.filter(department: \"Engineering\")} actor={@current_user}>\n      <:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"email\" filter>Email</:col>\n      <:col field=\"department.name\" filter>Department</:col>","title":"With Query Parameter (Advanced) - Cinder.Table","ref":"Cinder.Table.html#module-with-query-parameter-advanced"},{"type":"module","doc":"<:col field=\"title\" filter sort class=\"w-1/2\">\n        Title\n      </:col>\n      <:col field=\"artist.name\" filter sort>\n        Artist\n      </:col>\n      <:col field=\"genre\" filter={:select}>\n        Genre\n      </:col>","title":"Advanced Configuration - Cinder.Table","ref":"Cinder.Table.html#module-advanced-configuration"},{"type":"module","doc":"<!-- Admin interface with authorization and tenant -->\n      Ash.Query.for_read(:admin_read, %{}, actor: @actor, authorize?: @authorizing)\n        |> Ash.Query.set_tenant(@tenant)\n        |> Ash.Query.filter(active: true)}\n      actor={@actor}>\n      <:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"email\" filter>Email</:col>\n      <:col field=\"last_login\" sort>Last Login</:col>\n      <:col field=\"role\" filter={:select}>Role</:col>","title":"Complex Query Examples - Cinder.Table","ref":"Cinder.Table.html#module-complex-query-examples"},{"type":"module","doc":"<!-- Simple tenant support -->\n     \n      <:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"email\" filter>Email</:col>\n     \n\n    <!-- Using Ash scope (only actor and tenant are extracted) -->\n     \n      <:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"email\" filter>Email</:col>\n     \n\n    <!-- Custom scope struct -->\n     \n      <:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"email\" filter>Email</:col>\n     \n\n    <!-- Mixed usage (explicit overrides scope) -->\n     \n      <:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"email\" filter>Email</:col>","title":"Multi-Tenant Examples - Cinder.Table","ref":"Cinder.Table.html#module-multi-tenant-examples"},{"type":"module","doc":"- **Automatic type inference** from Ash resources\n- **Intelligent filtering** with automatic filter type detection\n- **URL state management** with browser back/forward support\n- **Relationship support** using dot notation (e.g., `artist.name`)\n- **Flexible theming** with built-in presets","title":"Features - Cinder.Table","ref":"Cinder.Table.html#module-features"},{"type":"function","doc":"Renders a data table with intelligent defaults.","title":"Cinder.Table.table/1","ref":"Cinder.Table.html#table/1"},{"type":"function","doc":"","title":"Attributes - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-attributes"},{"type":"function","doc":"- `resource` - Ash resource module to query (use either resource or query, not both)\n- `query` - Ash query to execute (use either resource or query, not both)","title":"Resource/Query (Choose One) - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-resource-query-choose-one"},{"type":"function","doc":"- `actor` - Actor for authorization (can be nil)","title":"Required - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-required"},{"type":"function","doc":"- `tenant` - Tenant for multi-tenant resources (default: nil)\n- `scope` - Ash scope containing actor and tenant (default: nil)","title":"Authorization & Tenancy - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-authorization-tenancy"},{"type":"function","doc":"- `id` - Component ID (defaults to \"cinder-table\")\n- `page_size` - Number of items per page (default: 25)\n- `theme` - Theme preset or custom theme map (default: \"default\")\n- `url_state` - URL state object from UrlSync.handle_params, or false to disable URL synchronization\n- `query_opts` - Additional query options for Ash (default: [])\n- `on_state_change` - Callback for state changes\n- `show_filters` - Show filter controls (default: auto-detect from columns)\n- `show_pagination` - Show pagination controls (default: true)\n- `loading_message` - Custom loading message\n- `empty_message` - Custom empty state message\n- `class` - Additional CSS classes","title":"Optional Configuration - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-optional-configuration"},{"type":"function","doc":"**Use `resource` for:**\n- Simple tables with default read actions\n- Getting started quickly\n- Standard use cases without custom requirements\n\n**Use `query` for:**\n- Custom read actions (e.g., `:active_users`, `:admin_only`)\n- Pre-filtering data with base filters\n- Custom authorization settings\n- Tenant-specific queries\n- Admin interfaces with complex requirements\n- Integration with existing Ash query pipelines","title":"When to Use Resource vs Query - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-when-to-use-resource-vs-query"},{"type":"function","doc":"The `:col` slot supports these attributes:\n\n- `field` (required) - Field name or relationship path (e.g., \"user.name\")\n- `filter` - Enable filtering (boolean or filter type atom)\n- `sort` - Enable sorting (boolean)\n- `class` - CSS classes for this column\n- `label` - Column header label (auto-generated from field name if not provided)\n\nFilter types: `:text`, `:select`, `:multi_select`, `:multi_checkboxes`, `:boolean`, `:date_range`, `:number_range`\n\n**Filter Type Selection:**\n- `:multi_select` - Modern tag-based interface with dropdown (default for array types)\n- `:multi_checkboxes` - Traditional checkbox interface for multiple selection","title":"Column Slot - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-column-slot"},{"type":"function","doc":"Column labels are automatically generated from field names using intelligent humanization:\n- `name` → \"Name\"\n- `email_address` → \"Email Address\"\n- `user.name` → \"User Name\"\n- `created_at` → \"Created At\"\n\nYou can override the auto-generated label by providing a `label` attribute.","title":"Column Labels - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-column-labels"},{"type":"function","doc":"Tables can be made interactive by providing a `row_click` function that will be\nexecuted when a row is clicked:\n\n      JS.navigate(~p\"/items/#{item.id}\") end}\n    >\n      <:col field=\"name\" filter sort>Name</:col>\n      <:col field=\"description\">Description</:col>\n     \n\nThe `row_click` function receives the row item as its argument and should return\na Phoenix.LiveView.JS command or similar action. When provided, rows will be\nstyled to indicate they are clickable with hover effects and cursor changes.","title":"Row Click Functionality - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-row-click-functionality"},{"type":"function","doc":"* `resource` (`:atom`) - The Ash resource to query (use either resource or query, not both). Defaults to `nil`.\n* `query` (`:any`) - The Ash query to execute (use either resource or query, not both). Defaults to `nil`.\n* `actor` (`:any`) - Actor for authorization. Defaults to `nil`.\n* `tenant` (`:any`) - Tenant for multi-tenant resources. Defaults to `nil`.\n* `scope` (`:any`) - Ash scope containing actor and tenant. Defaults to `nil`.\n* `id` (`:string`) - Unique identifier for the table. Defaults to `\"cinder-table\"`.\n* `page_size` (`:integer`) - Number of items per page. Defaults to `25`.\n* `theme` (`:any`) - Theme name or theme map. Defaults to `\"default\"`.\n* `url_state` (`:any`) - URL state object from UrlSync.handle_params, or false to disable. Defaults to `false`.\n* `query_opts` (`:list`) - Additional query options (load, select, etc.). Defaults to `[]`.\n* `on_state_change` (`:any`) - Custom state change handler. Defaults to `nil`.\n* `show_pagination` (`:boolean`) - Whether to show pagination controls. Defaults to `true`.\n* `show_filters` (`:boolean`) - Whether to show filter controls (auto-detected if nil). Defaults to `nil`.\n* `loading_message` (`:string`) - Message to show while loading. Defaults to `\"Loading...\"`.\n* `empty_message` (`:string`) - Message to show when no results. Defaults to `\"No results found\"`.\n* `class` (`:string`) - Additional CSS classes. Defaults to `\"\"`.\n* `row_click` (`:any`) - Function to call when a row is clicked. Receives the row item as argument. Defaults to `nil`.","title":"Attributes - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-attributes-1"},{"type":"function","doc":"* `col` (required) - Accepts attributes:\n\n  * `field` (`:string`) (required) - Field name (supports dot notation for relationships).\n  * `filter` (`:any`) - Enable filtering (true, false, or filter type atom).\n  * `filter_options` (`:list`) - Custom filter options (e.g., [options: [{\"Label\", \"value\"}]]).\n  * `sort` (`:boolean`) - Enable sorting.\n  * `label` (`:string`) - Custom column label (auto-generated if not provided).\n  * `class` (`:string`) - CSS classes for this column.","title":"Slots - Cinder.Table.table/1","ref":"Cinder.Table.html#table/1-slots"},{"type":"module","doc":"LiveComponent for interactive data tables with Ash query execution.\n\nHandles state management, data loading, and pagination for the table component.","title":"Cinder.Table.LiveComponent","ref":"Cinder.Table.LiveComponent.html"},{"type":"behaviour","doc":"Base behavior for Cinder filter implementations.\n\nDefines the common interface that all filter types must implement,\nalong with shared types and utility functions.","title":"Cinder.Filter","ref":"Cinder.Filter.html"},{"type":"behaviour","doc":"The most convenient way to create a custom filter is to use this module:\n\n    defmodule MyApp.Filters.Slider do\n      use Cinder.Filter\n\n      @impl true\n      def render(column, current_value, theme, assigns) do\n        filter_options = Map.get(column, :filter_options, [])\n        min_value = get_option(filter_options, :min, 0)\n        max_value = get_option(filter_options, :max, 100)\n        current = current_value || min_value\n\n        assigns = %{\n          column: column,\n          current_value: current,\n          min_value: min_value,\n          max_value: max_value,\n          theme: theme\n        }\n\n        ~H\"\"\"\n         \n           \n           {@current_value} \n         \n        \"\"\"\n      end\n\n      @impl true\n      def process(raw_value, column) when is_binary(raw_value) do\n        case Integer.parse(raw_value) do\n          {value, \"\"} ->\n            %{\n              type: :slider,\n              value: value,\n              operator: :less_than_or_equal\n            }\n          _ -> nil\n        end\n      end\n\n      def process(_raw_value, _column), do: nil\n\n      @impl true\n      def validate(%{type: :slider, value: value, operator: operator})\n          when is_integer(value) and is_atom(operator), do: true\n      def validate(_), do: false\n\n      @impl true\n      def default_options, do: [min: 0, max: 100, step: 1]\n\n      @impl true\n      def empty?(value) do\n        case value do\n          nil -> true\n          %{value: nil} -> true\n          _ -> false\n        end\n      end\n\n      @impl true\n      def build_query(query, field, filter_value) do\n        %{value: value} = filter_value\n        field_atom = String.to_atom(field)\n        Ash.Query.filter(query, ^ref(field_atom) <= ^value)\n      end\n    end\n\nThen register it in your configuration:\n\n    config :cinder, :filters, %{\n      slider: MyApp.Filters.Slider\n    }\n\nAnd call `Cinder.setup()` in your application start function to register\nall configured filters.\n\nThis automatically:\n- Adds the `@behaviour Cinder.Filter` declaration\n- Imports Phoenix.Component for HEEx templates\n- Imports helper functions from this module","title":"Quick Start - Cinder.Filter","ref":"Cinder.Filter.html#module-quick-start"},{"type":"behaviour","doc":"All custom filters must implement these callbacks:","title":"Required Callbacks - Cinder.Filter","ref":"Cinder.Filter.html#module-required-callbacks"},{"type":"behaviour","doc":"Renders the filter UI component.\n\n    @callback render(column :: map(), current_value :: any(), theme :: map(), assigns :: map()) :: Phoenix.LiveView.Rendered.t()\n\n- `column`: Column configuration with `field`, `label`, `filter_options`\n- `current_value`: Current filter value (nil if no filter applied)\n- `theme`: Theme configuration with CSS classes\n- `assigns`: Additional assigns from the parent component","title":"render/4 - Cinder.Filter","ref":"Cinder.Filter.html#module-render-4"},{"type":"behaviour","doc":"Processes raw form/URL input into structured filter data.\n\n    @callback process(raw_value :: any(), column :: map()) :: map() | nil\n\nMust return a map with `:type`, `:value`, `:operator` keys, or `nil`.","title":"process/2 - Cinder.Filter","ref":"Cinder.Filter.html#module-process-2"},{"type":"behaviour","doc":"Validates a processed filter value.\n\n    @callback validate(value :: any()) :: boolean()","title":"validate/1 - Cinder.Filter","ref":"Cinder.Filter.html#module-validate-1"},{"type":"behaviour","doc":"Returns default configuration options.\n\n    @callback default_options() :: keyword()","title":"default_options/0 - Cinder.Filter","ref":"Cinder.Filter.html#module-default_options-0"},{"type":"behaviour","doc":"Determines if a filter value is \"empty\" (no filtering applied).\n\n    @callback empty?(value :: any()) :: boolean()","title":"empty?/1 - Cinder.Filter","ref":"Cinder.Filter.html#module-empty-1"},{"type":"behaviour","doc":"**Critical for functionality!** Builds the Ash query for this filter.\n\n    @callback build_query(query :: Ash.Query.t(), field :: String.t(), filter_value :: map()) :: Ash.Query.t()","title":"build_query/3 - Cinder.Filter","ref":"Cinder.Filter.html#module-build_query-3"},{"type":"behaviour","doc":"","title":"Helper Functions - Cinder.Filter","ref":"Cinder.Filter.html#module-helper-functions"},{"type":"behaviour","doc":"Generates proper form field names for filter inputs:\n\n    ~H\"\"\"\n     \n    \"\"\"","title":"field_name/1 - Cinder.Filter","ref":"Cinder.Filter.html#module-field_name-1"},{"type":"behaviour","doc":"Safely extracts options from filter configuration with defaults:\n\n    filter_options = Map.get(column, :filter_options, [])\n    placeholder = get_option(filter_options, :placeholder, \"Enter text...\")","title":"get_option/3 - Cinder.Filter","ref":"Cinder.Filter.html#module-get_option-3"},{"type":"behaviour","doc":"","title":"Query Building Patterns - Cinder.Filter","ref":"Cinder.Filter.html#module-query-building-patterns"},{"type":"behaviour","doc":"def build_query(query, field, filter_value) do\n      %{value: value} = filter_value\n      field_atom = String.to_atom(field)\n      Ash.Query.filter(query, ^ref(field_atom) == ^value)\n    end","title":"Basic Field Filtering - Cinder.Filter","ref":"Cinder.Filter.html#module-basic-field-filtering"},{"type":"behaviour","doc":"Handle dot notation fields like \"user.name\":\n\n    def build_query(query, field, filter_value) do\n      %{value: value} = filter_value\n\n      if String.contains?(field, \".\") do\n        path_atoms = field |> String.split(\".\") |> Enum.map(&String.to_atom/1)\n        {rel_path, [field_atom]} = Enum.split(path_atoms, -1)\n\n        Ash.Query.filter(query, exists(^rel_path, ^ref(field_atom) == ^value))\n      else\n        field_atom = String.to_atom(field)\n        Ash.Query.filter(query, ^ref(field_atom) == ^value)\n      end\n    end","title":"Relationship Filtering - Cinder.Filter","ref":"Cinder.Filter.html#module-relationship-filtering"},{"type":"behaviour","doc":"1. **Always implement build_query/3** - This is what actually filters data\n2. **Handle edge cases in process/2** - Return nil for invalid input\n3. **Validate filter values** - Check structure and data types\n4. **Document your filters** - Include usage examples and options","title":"Best Practices - Cinder.Filter","ref":"Cinder.Filter.html#module-best-practices"},{"type":"callback","doc":"Builds query filters for this filter type.","title":"Cinder.Filter.build_query/3","ref":"Cinder.Filter.html#c:build_query/3"},{"type":"callback","doc":"- `query` - The Ash query to modify\n- `field` - The field name being filtered\n- `filter_value` - The processed filter value","title":"Parameters - Cinder.Filter.build_query/3","ref":"Cinder.Filter.html#c:build_query/3-parameters"},{"type":"callback","doc":"Modified Ash query with the filter applied","title":"Returns - Cinder.Filter.build_query/3","ref":"Cinder.Filter.html#c:build_query/3-returns"},{"type":"type","doc":"","title":"Cinder.Filter.column/0","ref":"Cinder.Filter.html#t:column/0"},{"type":"callback","doc":"Returns default options for this filter type.","title":"Cinder.Filter.default_options/0","ref":"Cinder.Filter.html#c:default_options/0"},{"type":"callback","doc":"Keyword list of default filter options","title":"Returns - Cinder.Filter.default_options/0","ref":"Cinder.Filter.html#c:default_options/0-returns"},{"type":"callback","doc":"Checks if a filter value is considered empty/inactive.","title":"Cinder.Filter.empty?/1","ref":"Cinder.Filter.html#c:empty?/1"},{"type":"callback","doc":"- `value` - Filter value to check","title":"Parameters - Cinder.Filter.empty?/1","ref":"Cinder.Filter.html#c:empty?/1-parameters"},{"type":"callback","doc":"Boolean indicating if the filter should be considered inactive","title":"Returns - Cinder.Filter.empty?/1","ref":"Cinder.Filter.html#c:empty?/1-returns"},{"type":"function","doc":"Generates a form field name for the given column key.","title":"Cinder.Filter.field_name/2","ref":"Cinder.Filter.html#field_name/2"},{"type":"type","doc":"","title":"Cinder.Filter.filter_options/0","ref":"Cinder.Filter.html#t:filter_options/0"},{"type":"type","doc":"","title":"Cinder.Filter.filter_value/0","ref":"Cinder.Filter.html#t:filter_value/0"},{"type":"function","doc":"Gets a nested value from filter options with a default.","title":"Cinder.Filter.get_option/3","ref":"Cinder.Filter.html#get_option/3"},{"type":"function","doc":"Checks if a filter has a meaningful value across all filter types.","title":"Cinder.Filter.has_filter_value?/1","ref":"Cinder.Filter.html#has_filter_value?/1"},{"type":"function","doc":"Converts an atom to human readable string.","title":"Cinder.Filter.humanize_atom/1","ref":"Cinder.Filter.html#humanize_atom/1"},{"type":"function","doc":"Converts a key to human readable string.","title":"Cinder.Filter.humanize_key/1","ref":"Cinder.Filter.html#humanize_key/1"},{"type":"function","doc":"Merges default options with provided options.","title":"Cinder.Filter.merge_options/2","ref":"Cinder.Filter.html#merge_options/2"},{"type":"callback","doc":"Processes raw form input into structured filter value.","title":"Cinder.Filter.process/2","ref":"Cinder.Filter.html#c:process/2"},{"type":"callback","doc":"- `raw_value` - Raw value from form submission\n- `column` - Column definition with filter configuration","title":"Parameters - Cinder.Filter.process/2","ref":"Cinder.Filter.html#c:process/2-parameters"},{"type":"callback","doc":"Structured filter value or nil if invalid","title":"Returns - Cinder.Filter.process/2","ref":"Cinder.Filter.html#c:process/2-returns"},{"type":"callback","doc":"Renders the filter input component for this filter type.","title":"Cinder.Filter.render/4","ref":"Cinder.Filter.html#c:render/4"},{"type":"callback","doc":"- `column` - Column definition with filter configuration\n- `current_value` - Current filter value\n- `theme` - Theme configuration for styling\n- `assigns` - Additional assigns (target, filter_values, etc.)","title":"Parameters - Cinder.Filter.render/4","ref":"Cinder.Filter.html#c:render/4-parameters"},{"type":"callback","doc":"HEEx template for the filter input","title":"Returns - Cinder.Filter.render/4","ref":"Cinder.Filter.html#c:render/4-returns"},{"type":"type","doc":"","title":"Cinder.Filter.theme/0","ref":"Cinder.Filter.html#t:theme/0"},{"type":"callback","doc":"Validates a filter value for this filter type.","title":"Cinder.Filter.validate/1","ref":"Cinder.Filter.html#c:validate/1"},{"type":"callback","doc":"- `value` - Filter value to validate","title":"Parameters - Cinder.Filter.validate/1","ref":"Cinder.Filter.html#c:validate/1-parameters"},{"type":"callback","doc":"Boolean indicating if value is valid","title":"Returns - Cinder.Filter.validate/1","ref":"Cinder.Filter.html#c:validate/1-returns"},{"type":"module","doc":"Debugging tools for custom filter development.\n\nThis module provides utilities to help developers debug and test their custom filters\nduring development.","title":"Cinder.Filter.Debug","ref":"Cinder.Filter.Debug.html"},{"type":"module","doc":"Enable debug mode in your configuration:\n\n    config :cinder, debug_filters: true\n\nThen use the debugging functions in your filter:\n\n    defmodule MyApp.Filters.CustomFilter do\n      use Cinder.Filter\n      import Cinder.Filter.Debug\n\n      @impl true\n      def process(raw_value, column) do\n        debug_step(\"Processing input\", %{raw_value: raw_value, column: column})\n\n        result = # ... your processing logic\n\n        debug_step(\"Process result\", %{result: result})\n        result\n      end\n    end","title":"Usage - Cinder.Filter.Debug","ref":"Cinder.Filter.Debug.html#module-usage"},{"type":"function","doc":"Logs filter callback execution with timing.","title":"Cinder.Filter.Debug.debug_callback/2","ref":"Cinder.Filter.Debug.html#debug_callback/2"},{"type":"function","doc":"debug_callback(\"process/2\", fn ->\n      # Your callback logic here\n      process_implementation(raw_value, column)\n    end)","title":"Examples - Cinder.Filter.Debug.debug_callback/2","ref":"Cinder.Filter.Debug.html#debug_callback/2-examples"},{"type":"function","doc":"Runs a comprehensive filter test suite.\n\nTests all callbacks with various inputs and logs results.","title":"Cinder.Filter.Debug.debug_comprehensive_test/1","ref":"Cinder.Filter.Debug.html#debug_comprehensive_test/1"},{"type":"function","doc":"debug_comprehensive_test(MyApp.Filters.Slider)","title":"Examples - Cinder.Filter.Debug.debug_comprehensive_test/1","ref":"Cinder.Filter.Debug.html#debug_comprehensive_test/1-examples"},{"type":"function","doc":"Checks if debug mode is enabled.","title":"Cinder.Filter.Debug.debug_enabled?/0","ref":"Cinder.Filter.Debug.html#debug_enabled?/0"},{"type":"function","doc":"Validates and logs filter processing pipeline.\n\nUseful for debugging the complete flow from raw input to final filter.","title":"Cinder.Filter.Debug.debug_pipeline/4","ref":"Cinder.Filter.Debug.html#debug_pipeline/4"},{"type":"function","doc":"debug_pipeline(\"MyFilter\", raw_value, column, fn ->\n      MyFilter.process(raw_value, column)\n    end)","title":"Examples - Cinder.Filter.Debug.debug_pipeline/4","ref":"Cinder.Filter.Debug.html#debug_pipeline/4-examples"},{"type":"function","doc":"Analyzes query building performance and logs the results.","title":"Cinder.Filter.Debug.debug_query_building/3","ref":"Cinder.Filter.Debug.html#debug_query_building/3"},{"type":"function","doc":"debug_query_building(MyApp.Filters.Slider, \"price\", %{\n      type: :slider,\n      value: 100,\n      operator: :less_than_or_equal\n    })","title":"Examples - Cinder.Filter.Debug.debug_query_building/3","ref":"Cinder.Filter.Debug.html#debug_query_building/3-examples"},{"type":"function","doc":"Logs render performance and output size.","title":"Cinder.Filter.Debug.debug_render_performance/5","ref":"Cinder.Filter.Debug.html#debug_render_performance/5"},{"type":"function","doc":"debug_render_performance(MyApp.Filters.Slider, column, current_value, theme, assigns)","title":"Examples - Cinder.Filter.Debug.debug_render_performance/5","ref":"Cinder.Filter.Debug.html#debug_render_performance/5-examples"},{"type":"function","doc":"Logs a debug step with context information.\n\nOnly logs when debug_filters is enabled in configuration.","title":"Cinder.Filter.Debug.debug_step/2","ref":"Cinder.Filter.Debug.html#debug_step/2"},{"type":"function","doc":"debug_step(\"Validating input\", %{input: \"test\", step: 1})","title":"Examples - Cinder.Filter.Debug.debug_step/2","ref":"Cinder.Filter.Debug.html#debug_step/2-examples"},{"type":"function","doc":"Tests filter processing with sample inputs and logs results.\n\nUseful for quick testing during development.","title":"Cinder.Filter.Debug.debug_test_inputs/2","ref":"Cinder.Filter.Debug.html#debug_test_inputs/2"},{"type":"function","doc":"debug_test_inputs(MyApp.Filters.Slider, [\n      {\"50\", %{filter_options: [min: 0, max: 100]}},\n      {\"invalid\", %{filter_options: []}},\n      {\"\", %{filter_options: []}}\n    ])","title":"Examples - Cinder.Filter.Debug.debug_test_inputs/2","ref":"Cinder.Filter.Debug.html#debug_test_inputs/2-examples"},{"type":"function","doc":"Validates a filter's callbacks and logs any issues.\n\nUseful during development to ensure all callbacks are properly implemented.","title":"Cinder.Filter.Debug.debug_validate_filter/1","ref":"Cinder.Filter.Debug.html#debug_validate_filter/1"},{"type":"function","doc":"debug_validate_filter(MyApp.Filters.CustomFilter)","title":"Examples - Cinder.Filter.Debug.debug_validate_filter/1","ref":"Cinder.Filter.Debug.html#debug_validate_filter/1-examples"},{"type":"function","doc":"Disables debug mode for the current session.","title":"Cinder.Filter.Debug.disable_debug/0","ref":"Cinder.Filter.Debug.html#disable_debug/0"},{"type":"function","doc":"iex> Cinder.Filter.Debug.disable_debug()\n    :ok","title":"Examples - Cinder.Filter.Debug.disable_debug/0","ref":"Cinder.Filter.Debug.html#disable_debug/0-examples"},{"type":"function","doc":"Enables debug mode for the current session.\n\nUseful in IEx for temporary debugging.","title":"Cinder.Filter.Debug.enable_debug/0","ref":"Cinder.Filter.Debug.html#enable_debug/0"},{"type":"function","doc":"iex> Cinder.Filter.Debug.enable_debug()\n    :ok","title":"Examples - Cinder.Filter.Debug.enable_debug/0","ref":"Cinder.Filter.Debug.html#enable_debug/0-examples"},{"type":"module","doc":"Helper functions for building and validating custom filters.\n\nThis module provides common patterns and utilities that custom filter\ndevelopers can use to simplify their implementations.","title":"Cinder.Filter.Helpers","ref":"Cinder.Filter.Helpers.html"},{"type":"module","doc":"defmodule MyApp.Filters.CustomFilter do\n      use Cinder.Filter\n      import Cinder.Filter.Helpers\n\n      @impl true\n      def process(raw_value, column) do\n        with {:ok, trimmed} <- validate_string_input(raw_value),\n             {:ok, parsed} <- parse_custom_value(trimmed) do\n          build_filter(:my_filter, parsed, :equals)\n        else\n          _ -> nil\n        end\n      end\n    end","title":"Usage - Cinder.Filter.Helpers","ref":"Cinder.Filter.Helpers.html#module-usage"},{"type":"function","doc":"Builds a relationship-aware Ash query filter.\n\nHandles both direct fields and relationship fields using dot notation.","title":"Cinder.Filter.Helpers.build_ash_filter/4","ref":"Cinder.Filter.Helpers.html#build_ash_filter/4"},{"type":"function","doc":"build_ash_filter(query, \"name\", \"John\", :equals)\n    build_ash_filter(query, \"user.name\", \"John\", :equals)","title":"Examples - Cinder.Filter.Helpers.build_ash_filter/4","ref":"Cinder.Filter.Helpers.html#build_ash_filter/4-examples"},{"type":"function","doc":"Builds a standard filter map.","title":"Cinder.Filter.Helpers.build_filter/4","ref":"Cinder.Filter.Helpers.html#build_filter/4"},{"type":"function","doc":"iex> build_filter(:my_filter, \"value\", :equals)\n    %{type: :my_filter, value: \"value\", operator: :equals}\n\n    iex> build_filter(:slider, 50, :less_than_or_equal, case_sensitive: false)\n    %{type: :slider, value: 50, operator: :less_than_or_equal, case_sensitive: false}","title":"Examples - Cinder.Filter.Helpers.build_filter/4","ref":"Cinder.Filter.Helpers.html#build_filter/4-examples"},{"type":"function","doc":"Debug helper for filter development.\n\nLogs filter processing information when debug is enabled.","title":"Cinder.Filter.Helpers.debug_filter/3","ref":"Cinder.Filter.Helpers.html#debug_filter/3"},{"type":"function","doc":"debug_filter(\"MyFilter\", \"processing input\", %{input: \"test\"})","title":"Examples - Cinder.Filter.Helpers.debug_filter/3","ref":"Cinder.Filter.Helpers.html#debug_filter/3-examples"},{"type":"function","doc":"Extracts filter options with type safety.","title":"Cinder.Filter.Helpers.extract_option/3","ref":"Cinder.Filter.Helpers.html#extract_option/3"},{"type":"function","doc":"iex> extract_option([min: 0, max: 100], :min, 50)\n    0\n\n    iex> extract_option([], :min, 50)\n    50\n\n    iex> extract_option(%{min: 0, max: 100}, :min, 50)\n    0","title":"Examples - Cinder.Filter.Helpers.extract_option/3","ref":"Cinder.Filter.Helpers.html#extract_option/3-examples"},{"type":"function","doc":"Common empty value check for most filter types.","title":"Cinder.Filter.Helpers.is_empty_value?/1","ref":"Cinder.Filter.Helpers.html#is_empty_value?/1"},{"type":"function","doc":"iex> is_empty_value?(nil)\n    true\n\n    iex> is_empty_value?(\"\")\n    true\n\n    iex> is_empty_value?([])\n    true\n\n    iex> is_empty_value?(%{value: nil})\n    true\n\n    iex> is_empty_value?(\"test\")\n    false","title":"Examples - Cinder.Filter.Helpers.is_empty_value?/1","ref":"Cinder.Filter.Helpers.html#is_empty_value?/1-examples"},{"type":"function","doc":"Validates and parses comma-separated values.","title":"Cinder.Filter.Helpers.validate_csv_input/2","ref":"Cinder.Filter.Helpers.html#validate_csv_input/2"},{"type":"function","doc":"iex> validate_csv_input(\"a,b,c\")\n    {:ok, [\"a\", \"b\", \"c\"]}\n\n    iex> validate_csv_input(\"a, b , c \", trim: true)\n    {:ok, [\"a\", \"b\", \"c\"]}\n\n    iex> validate_csv_input(\"\", min_length: 1)\n    {:error, :empty}","title":"Examples - Cinder.Filter.Helpers.validate_csv_input/2","ref":"Cinder.Filter.Helpers.html#validate_csv_input/2-examples"},{"type":"function","doc":"Validates date input in ISO 8601 format.","title":"Cinder.Filter.Helpers.validate_date_input/1","ref":"Cinder.Filter.Helpers.html#validate_date_input/1"},{"type":"function","doc":"iex> validate_date_input(\"2023-12-25\")\n    {:ok, ~D[2023-12-25]}\n\n    iex> validate_date_input(\"invalid-date\")\n    {:error, :invalid}","title":"Examples - Cinder.Filter.Helpers.validate_date_input/1","ref":"Cinder.Filter.Helpers.html#validate_date_input/1-examples"},{"type":"function","doc":"Validates that a module properly implements the Cinder.Filter behaviour.","title":"Cinder.Filter.Helpers.validate_filter_implementation/1","ref":"Cinder.Filter.Helpers.html#validate_filter_implementation/1"},{"type":"function","doc":"iex> validate_filter_implementation(MyApp.Filters.ValidFilter)\n    {:ok, \"Filter implementation is valid\"}\n\n    iex> validate_filter_implementation(InvalidModule)\n    {:error, [\"Missing callback: render/4\", \"Missing callback: process/2\"]}","title":"Examples - Cinder.Filter.Helpers.validate_filter_implementation/1","ref":"Cinder.Filter.Helpers.html#validate_filter_implementation/1-examples"},{"type":"function","doc":"Validates a filter structure has required fields.","title":"Cinder.Filter.Helpers.validate_filter_structure/1","ref":"Cinder.Filter.Helpers.html#validate_filter_structure/1"},{"type":"function","doc":"iex> validate_filter_structure(%{type: :text, value: \"test\", operator: :equals})\n    {:ok, %{type: :text, value: \"test\", operator: :equals}}\n\n    iex> validate_filter_structure(%{type: :text, value: \"test\"})\n    {:error, :missing_operator}","title":"Examples - Cinder.Filter.Helpers.validate_filter_structure/1","ref":"Cinder.Filter.Helpers.html#validate_filter_structure/1-examples"},{"type":"function","doc":"Validates float input with optional min/max bounds.","title":"Cinder.Filter.Helpers.validate_float_input/2","ref":"Cinder.Filter.Helpers.html#validate_float_input/2"},{"type":"function","doc":"iex> validate_float_input(\"42.5\")\n    {:ok, 42.5}\n\n    iex> validate_float_input(\"42.5\", min: 0.0, max: 100.0)\n    {:ok, 42.5}\n\n    iex> validate_float_input(\"150.0\", max: 100.0)\n    {:error, :out_of_bounds}\n\n    iex> validate_float_input(\"abc\")\n    {:error, :invalid}","title":"Examples - Cinder.Filter.Helpers.validate_float_input/2","ref":"Cinder.Filter.Helpers.html#validate_float_input/2-examples"},{"type":"function","doc":"Validates hex color input.","title":"Cinder.Filter.Helpers.validate_hex_color_input/1","ref":"Cinder.Filter.Helpers.html#validate_hex_color_input/1"},{"type":"function","doc":"iex> validate_hex_color_input(\"#FF0000\")\n    {:ok, \"#ff0000\"}\n\n    iex> validate_hex_color_input(\"#fff\")\n    {:error, :invalid}\n\n    iex> validate_hex_color_input(\"red\")\n    {:error, :invalid}","title":"Examples - Cinder.Filter.Helpers.validate_hex_color_input/1","ref":"Cinder.Filter.Helpers.html#validate_hex_color_input/1-examples"},{"type":"function","doc":"Validates integer input with optional min/max bounds.","title":"Cinder.Filter.Helpers.validate_integer_input/2","ref":"Cinder.Filter.Helpers.html#validate_integer_input/2"},{"type":"function","doc":"iex> validate_integer_input(\"42\")\n    {:ok, 42}\n\n    iex> validate_integer_input(\"42\", min: 0, max: 100)\n    {:ok, 42}\n\n    iex> validate_integer_input(\"150\", max: 100)\n    {:error, :out_of_bounds}\n\n    iex> validate_integer_input(\"abc\")\n    {:error, :invalid}","title":"Examples - Cinder.Filter.Helpers.validate_integer_input/2","ref":"Cinder.Filter.Helpers.html#validate_integer_input/2-examples"},{"type":"function","doc":"Validates operator is in allowed list.","title":"Cinder.Filter.Helpers.validate_operator/2","ref":"Cinder.Filter.Helpers.html#validate_operator/2"},{"type":"function","doc":"iex> validate_operator(:equals, [:equals, :contains])\n    {:ok, :equals}\n\n    iex> validate_operator(:invalid, [:equals, :contains])\n    {:error, :invalid_operator}","title":"Examples - Cinder.Filter.Helpers.validate_operator/2","ref":"Cinder.Filter.Helpers.html#validate_operator/2-examples"},{"type":"function","doc":"Validates and trims string input, returning error for empty strings.","title":"Cinder.Filter.Helpers.validate_string_input/1","ref":"Cinder.Filter.Helpers.html#validate_string_input/1"},{"type":"function","doc":"iex> validate_string_input(\"  hello  \")\n    {:ok, \"hello\"}\n\n    iex> validate_string_input(\"\")\n    {:error, :empty}\n\n    iex> validate_string_input(nil)\n    {:error, :invalid}","title":"Examples - Cinder.Filter.Helpers.validate_string_input/1","ref":"Cinder.Filter.Helpers.html#validate_string_input/1-examples"},{"type":"module","doc":"Boolean filter implementation for Cinder tables.\n\nProvides boolean filtering with radio button inputs for true/false/all options.","title":"Cinder.Filters.Boolean","ref":"Cinder.Filters.Boolean.html"},{"type":"module","doc":"Date range filter implementation for Cinder tables.\n\nProvides date range filtering with from/to date inputs.\nSupports both date-only and datetime filtering via the `include_time` option.","title":"Cinder.Filters.DateRange","ref":"Cinder.Filters.DateRange.html"},{"type":"module","doc":"Multi-checkbox filter implementation for Cinder tables.\n\nProvides multiple selection filtering with checkbox inputs for each option.\nThis is the traditional checkbox-based interface for selecting multiple values.","title":"Cinder.Filters.MultiCheckboxes","ref":"Cinder.Filters.MultiCheckboxes.html"},{"type":"module","doc":"Multi-select tag-based filter implementation for Cinder tables.\n\nProvides multiple selection filtering with a modern dropdown + tags interface.\nSelected items are displayed as removable tags with a dropdown for adding new selections.\nUses Phoenix LiveView patterns for interactivity.","title":"Cinder.Filters.MultiSelect","ref":"Cinder.Filters.MultiSelect.html"},{"type":"function","doc":"Handles toggling an option in the multi-select filter.\n\nThis function should be called from the parent LiveView/LiveComponent\nto handle the \"toggle_multiselect_option\" event.","title":"Cinder.Filters.MultiSelect.handle_toggle_option/3","ref":"Cinder.Filters.MultiSelect.html#handle_toggle_option/3"},{"type":"module","doc":"Number range filter implementation for Cinder tables.\n\nProvides numeric range filtering with min/max number inputs.","title":"Cinder.Filters.NumberRange","ref":"Cinder.Filters.NumberRange.html"},{"type":"module","doc":"Registry for managing available filter types in Cinder.\n\nProvides centralized registration and discovery of filter implementations,\nalong with default filter type inference based on data types.","title":"Cinder.Filters.Registry","ref":"Cinder.Filters.Registry.html"},{"type":"function","doc":"Returns a map of all registered filter types to their implementing modules.","title":"Cinder.Filters.Registry.all_filters/0","ref":"Cinder.Filters.Registry.html#all_filters/0"},{"type":"function","doc":"Gets all filters including custom registered ones.\n\nCustom filters take precedence over built-in filters if there's a naming conflict\n(though registration prevents this from happening).","title":"Cinder.Filters.Registry.all_filters_with_custom/0","ref":"Cinder.Filters.Registry.html#all_filters_with_custom/0"},{"type":"function","doc":"Map of all filter types to their implementing modules","title":"Returns - Cinder.Filters.Registry.all_filters_with_custom/0","ref":"Cinder.Filters.Registry.html#all_filters_with_custom/0-returns"},{"type":"function","doc":"iex> Cinder.Filters.Registry.all_filters_with_custom()\n    %{\n      text: Cinder.Filters.Text,\n      select: Cinder.Filters.Select,\n      slider: MyApp.Filters.Slider\n    }","title":"Examples - Cinder.Filters.Registry.all_filters_with_custom/0","ref":"Cinder.Filters.Registry.html#all_filters_with_custom/0-examples"},{"type":"function","doc":"Checks if a filter type is a custom (user-registered) filter.","title":"Cinder.Filters.Registry.custom_filter?/1","ref":"Cinder.Filters.Registry.html#custom_filter?/1"},{"type":"function","doc":"- `filter_type` - Filter type atom to check","title":"Parameters - Cinder.Filters.Registry.custom_filter?/1","ref":"Cinder.Filters.Registry.html#custom_filter?/1-parameters"},{"type":"function","doc":"Boolean indicating if the filter type is custom","title":"Returns - Cinder.Filters.Registry.custom_filter?/1","ref":"Cinder.Filters.Registry.html#custom_filter?/1-returns"},{"type":"function","doc":"iex> Cinder.Filters.Registry.custom_filter?(:slider)\n    true\n\n    iex> Cinder.Filters.Registry.custom_filter?(:text)\n    false","title":"Examples - Cinder.Filters.Registry.custom_filter?/1","ref":"Cinder.Filters.Registry.html#custom_filter?/1-examples"},{"type":"function","doc":"Gets default filter options for a given filter type.","title":"Cinder.Filters.Registry.default_options/2","ref":"Cinder.Filters.Registry.html#default_options/2"},{"type":"function","doc":"- `filter_type` - Filter type atom\n- `column_key` - Column key for context (optional)","title":"Parameters - Cinder.Filters.Registry.default_options/2","ref":"Cinder.Filters.Registry.html#default_options/2-parameters"},{"type":"function","doc":"Keyword list of default options for the filter type","title":"Returns - Cinder.Filters.Registry.default_options/2","ref":"Cinder.Filters.Registry.html#default_options/2-returns"},{"type":"function","doc":"Returns a list of all registered filter type atoms.","title":"Cinder.Filters.Registry.filter_types/0","ref":"Cinder.Filters.Registry.html#filter_types/0"},{"type":"function","doc":"iex> Cinder.Filters.Registry.filter_types()\n    [:text, :select, :multi_select, :multi_checkboxes, :date_range, :number_range, :boolean]","title":"Examples - Cinder.Filters.Registry.filter_types/0","ref":"Cinder.Filters.Registry.html#filter_types/0-examples"},{"type":"function","doc":"Gets the implementing module for a filter type.\n\nChecks both built-in and custom registered filters.","title":"Cinder.Filters.Registry.get_filter/1","ref":"Cinder.Filters.Registry.html#get_filter/1"},{"type":"function","doc":"iex> Cinder.Filters.Registry.get_filter(:text)\n    Cinder.Filters.Text\n\n    iex> Cinder.Filters.Registry.get_filter(:slider)\n    MyApp.Filters.Slider\n\n    iex> Cinder.Filters.Registry.get_filter(:unknown)\n    nil","title":"Examples - Cinder.Filters.Registry.get_filter/1","ref":"Cinder.Filters.Registry.html#get_filter/1-examples"},{"type":"function","doc":"Infers the appropriate filter type based on Ash resource attribute.","title":"Cinder.Filters.Registry.infer_filter_type/2","ref":"Cinder.Filters.Registry.html#infer_filter_type/2"},{"type":"function","doc":"- `attribute` - Ash resource attribute definition\n- `column_key` - Column key for context","title":"Parameters - Cinder.Filters.Registry.infer_filter_type/2","ref":"Cinder.Filters.Registry.html#infer_filter_type/2-parameters"},{"type":"function","doc":"Atom representing the inferred filter type","title":"Returns - Cinder.Filters.Registry.infer_filter_type/2","ref":"Cinder.Filters.Registry.html#infer_filter_type/2-returns"},{"type":"function","doc":"Lists all registered custom filter types.","title":"Cinder.Filters.Registry.list_custom_filters/0","ref":"Cinder.Filters.Registry.html#list_custom_filters/0"},{"type":"function","doc":"Map of custom filter types to their implementing modules","title":"Returns - Cinder.Filters.Registry.list_custom_filters/0","ref":"Cinder.Filters.Registry.html#list_custom_filters/0-returns"},{"type":"function","doc":"iex> Cinder.Filters.Registry.list_custom_filters()\n    %{slider: MyApp.Filters.Slider, color_picker: MyApp.Filters.ColorPicker}","title":"Examples - Cinder.Filters.Registry.list_custom_filters/0","ref":"Cinder.Filters.Registry.html#list_custom_filters/0-examples"},{"type":"function","doc":"","title":"Cinder.Filters.Registry.register_config_filter/2","ref":"Cinder.Filters.Registry.html#register_config_filter/2"},{"type":"function","doc":"Registers custom filters from application configuration.\n\nThis function should be called during application startup to register\nfilters defined in config files.","title":"Cinder.Filters.Registry.register_config_filters/0","ref":"Cinder.Filters.Registry.html#register_config_filters/0"},{"type":"function","doc":"config :cinder, :filters, %{\n      slider: MyApp.Filters.Slider,\n      color_picker: MyApp.Filters.ColorPicker\n    }","title":"Configuration - Cinder.Filters.Registry.register_config_filters/0","ref":"Cinder.Filters.Registry.html#register_config_filters/0-configuration"},{"type":"function","doc":":ok if all filters registered successfully, {:error, [reasons]} if any failed","title":"Returns - Cinder.Filters.Registry.register_config_filters/0","ref":"Cinder.Filters.Registry.html#register_config_filters/0-returns"},{"type":"function","doc":"Checks if a filter type is registered.\n\nChecks both built-in and custom registered filters.","title":"Cinder.Filters.Registry.registered?/1","ref":"Cinder.Filters.Registry.html#registered?/1"},{"type":"function","doc":"iex> Cinder.Filters.Registry.registered?(:text)\n    true\n\n    iex> Cinder.Filters.Registry.registered?(:slider)\n    true\n\n    iex> Cinder.Filters.Registry.registered?(:unknown)\n    false","title":"Examples - Cinder.Filters.Registry.registered?/1","ref":"Cinder.Filters.Registry.html#registered?/1-examples"},{"type":"function","doc":"Validates that all registered custom filters are properly implemented.\n\nThis function can be called at application startup to ensure all custom filters\nare valid and will work correctly.","title":"Cinder.Filters.Registry.validate_custom_filters/0","ref":"Cinder.Filters.Registry.html#validate_custom_filters/0"},{"type":"function","doc":":ok if all filters are valid, {:error, [reasons]} if any are invalid","title":"Returns - Cinder.Filters.Registry.validate_custom_filters/0","ref":"Cinder.Filters.Registry.html#validate_custom_filters/0-returns"},{"type":"function","doc":"iex> Cinder.Filters.Registry.validate_custom_filters()\n    :ok\n\n    iex> Cinder.Filters.Registry.validate_custom_filters()\n    {:error, [\"Module MyApp.Filters.BrokenSlider does not implement required function render/4\"]}","title":"Examples - Cinder.Filters.Registry.validate_custom_filters/0","ref":"Cinder.Filters.Registry.html#validate_custom_filters/0-examples"},{"type":"module","doc":"Select dropdown filter implementation for Cinder tables.\n\nProvides single-select filtering with configurable options and prompts.","title":"Cinder.Filters.Select","ref":"Cinder.Filters.Select.html"},{"type":"module","doc":"Text filter implementation for Cinder tables.\n\nProvides text-based filtering with support for different operators\nlike contains, equals, starts_with, etc.","title":"Cinder.Filters.Text","ref":"Cinder.Filters.Text.html"},{"type":"module","doc":"Theme properties for filter components.\n\nThis module defines all the theme properties that can be customized\nfor the filter system, including the filter container, inputs, and\nall filter type specific styling.","title":"Cinder.Components.Filters","ref":"Cinder.Components.Filters.html"},{"type":"function","doc":"Returns the default theme values for filter properties.\nProvides only the bare minimum classes needed for usability.","title":"Cinder.Components.Filters.default_theme/0","ref":"Cinder.Components.Filters.html#default_theme/0"},{"type":"function","doc":"Returns all theme properties available for filter components.","title":"Cinder.Components.Filters.theme_properties/0","ref":"Cinder.Components.Filters.html#theme_properties/0"},{"type":"function","doc":"Validates that a theme property key is valid for this component.","title":"Cinder.Components.Filters.valid_property?/1","ref":"Cinder.Components.Filters.html#valid_property?/1"},{"type":"module","doc":"Theme properties for loading components.\n\nThis module defines all the theme properties that can be customized\nfor loading indicators, spinners, and loading states throughout\nthe table component.","title":"Cinder.Components.Loading","ref":"Cinder.Components.Loading.html"},{"type":"function","doc":"Returns the default theme values for loading properties.\nProvides only the bare minimum classes needed for usability.","title":"Cinder.Components.Loading.default_theme/0","ref":"Cinder.Components.Loading.html#default_theme/0"},{"type":"function","doc":"Returns all theme properties available for loading components.","title":"Cinder.Components.Loading.theme_properties/0","ref":"Cinder.Components.Loading.html#theme_properties/0"},{"type":"function","doc":"Validates that a theme property key is valid for this component.","title":"Cinder.Components.Loading.valid_property?/1","ref":"Cinder.Components.Loading.html#valid_property?/1"},{"type":"module","doc":"Theme properties for pagination components.\n\nThis module defines all the theme properties that can be customized\nfor the pagination system, including the pagination wrapper, buttons,\nand info displays.","title":"Cinder.Components.Pagination","ref":"Cinder.Components.Pagination.html"},{"type":"function","doc":"Returns the default theme values for pagination properties.\nProvides only the bare minimum classes needed for usability.","title":"Cinder.Components.Pagination.default_theme/0","ref":"Cinder.Components.Pagination.html#default_theme/0"},{"type":"function","doc":"Returns all theme properties available for pagination components.","title":"Cinder.Components.Pagination.theme_properties/0","ref":"Cinder.Components.Pagination.html#theme_properties/0"},{"type":"function","doc":"Validates that a theme property key is valid for this component.","title":"Cinder.Components.Pagination.valid_property?/1","ref":"Cinder.Components.Pagination.html#valid_property?/1"},{"type":"module","doc":"Theme properties for sorting components.\n\nThis module defines all the theme properties that can be customized\nfor the sorting system, including sort indicators, icons, and\ninteractive elements.","title":"Cinder.Components.Sorting","ref":"Cinder.Components.Sorting.html"},{"type":"function","doc":"Returns the default theme values for sorting properties.\nProvides only the bare minimum classes needed for usability.","title":"Cinder.Components.Sorting.default_theme/0","ref":"Cinder.Components.Sorting.html#default_theme/0"},{"type":"function","doc":"Returns all theme properties available for sorting components.","title":"Cinder.Components.Sorting.theme_properties/0","ref":"Cinder.Components.Sorting.html#theme_properties/0"},{"type":"function","doc":"Validates that a theme property key is valid for this component.","title":"Cinder.Components.Sorting.valid_property?/1","ref":"Cinder.Components.Sorting.html#valid_property?/1"},{"type":"module","doc":"Theme properties for the main table component.\n\nThis module defines all the theme properties that can be customized\nfor the table structure itself, including the container, table element,\nheaders, rows, and cells.","title":"Cinder.Components.Table","ref":"Cinder.Components.Table.html"},{"type":"function","doc":"Returns the default theme values for table properties.\nProvides only the bare minimum classes needed for usability.","title":"Cinder.Components.Table.default_theme/0","ref":"Cinder.Components.Table.html#default_theme/0"},{"type":"function","doc":"Returns all theme properties available for the table component.","title":"Cinder.Components.Table.theme_properties/0","ref":"Cinder.Components.Table.html#theme_properties/0"},{"type":"function","doc":"Validates that a theme property key is valid for this component.","title":"Cinder.Components.Table.valid_property?/1","ref":"Cinder.Components.Table.html#valid_property?/1"},{"type":"module","doc":"Theme management for Cinder table components.\n\nProvides default themes and utilities for merging custom theme configurations.\nAlso supports the new Spark DSL for defining modular themes.","title":"Cinder.Theme","ref":"Cinder.Theme.html"},{"type":"module","doc":"# Using built-in themes\n    theme = Cinder.Theme.merge(\"modern\")\n\n    # Using application configuration for default theme\n    # config/config.exs\n    config :cinder, default_theme: \"modern\"","title":"Basic Usage - Cinder.Theme","ref":"Cinder.Theme.html#module-basic-usage"},{"type":"module","doc":"defmodule MyApp.CustomTheme do\n      use Cinder.Theme\n\n      override Cinder.Components.Table do\n        set :container_class, \"my-custom-table-container\"\n        set :row_class, \"my-custom-row hover:bg-blue-50\"\n      end\n    end\n\n    # Use in config\n    config :cinder, default_theme: MyApp.CustomTheme\n\n    # Or use directly\n    theme = Cinder.Theme.merge(MyApp.CustomTheme)","title":"Advanced Usage (DSL-based themes) - Cinder.Theme","ref":"Cinder.Theme.html#module-advanced-usage-dsl-based-themes"},{"type":"module","doc":"You can set a default theme for all Cinder tables in your application configuration:\n\n    # config/config.exs\n    config :cinder, default_theme: \"modern\"\n\n    # Or use a custom theme module\n    config :cinder, default_theme: MyApp.CustomTheme\n\nIndividual tables can still override the configured default:\n\n     \n      <!-- This table uses \"dark\" theme, ignoring the configured default -->","title":"Configuration - Cinder.Theme","ref":"Cinder.Theme.html#module-configuration"},{"type":"function","doc":"Gets all available theme properties across all components.","title":"Cinder.Theme.all_theme_properties/0","ref":"Cinder.Theme.html#all_theme_properties/0"},{"type":"function","doc":"Gets the complete default theme by merging all component defaults.","title":"Cinder.Theme.complete_default/0","ref":"Cinder.Theme.html#complete_default/0"},{"type":"function","doc":"Returns the default theme configuration.","title":"Cinder.Theme.default/0","ref":"Cinder.Theme.html#default/0"},{"type":"function","doc":"Gets the configured default theme from application configuration.\n\nReturns the theme configured via `config :cinder, default_theme: ...`\nor falls back to \"default\" if no configuration is set.","title":"Cinder.Theme.get_default_theme/0","ref":"Cinder.Theme.html#get_default_theme/0"},{"type":"function","doc":"# With configuration\n    Application.put_env(:cinder, :default_theme, \"modern\")\n    Cinder.Theme.get_default_theme()\n    #=> returns modern theme configuration\n\n    # Without configuration\n    Cinder.Theme.get_default_theme()\n    #=> returns \"default\" theme configuration","title":"Examples - Cinder.Theme.get_default_theme/0","ref":"Cinder.Theme.html#get_default_theme/0-examples"},{"type":"function","doc":"Merges a theme configuration with the default theme.","title":"Cinder.Theme.merge/1","ref":"Cinder.Theme.html#merge/1"},{"type":"function","doc":"iex> Cinder.Theme.merge(\"modern\")\n    %{container_class: \"bg-white shadow-lg rounded-xl border border-gray-100 overflow-hidden\", ...}\n\n    iex> Cinder.Theme.merge(MyApp.CustomTheme)\n    %{container_class: \"custom-container\", ...}","title":"Examples - Cinder.Theme.merge/1","ref":"Cinder.Theme.html#merge/1-examples"},{"type":"function","doc":"Returns a list of available theme presets.","title":"Cinder.Theme.presets/0","ref":"Cinder.Theme.html#presets/0"},{"type":"type","doc":"","title":"Cinder.Theme.theme/0","ref":"Cinder.Theme.html#t:theme/0"},{"type":"function","doc":"Validates a theme configuration.\n\nReturns :ok if the theme is valid, or {:error, reason} if invalid.","title":"Cinder.Theme.validate/1","ref":"Cinder.Theme.html#validate/1"},{"type":"behaviour","doc":"Behaviour for Cinder theme modules.\n\nThis behaviour defines the interface that all theme modules must implement,\nwhether they use the DSL or are simple theme modules.","title":"Cinder.Theme.Behaviour","ref":"Cinder.Theme.Behaviour.html"},{"type":"callback","doc":"Resolves the theme configuration into a map of CSS classes.\n\nReturns a map where keys are theme property atoms and values are CSS class strings.","title":"Cinder.Theme.Behaviour.resolve_theme/0","ref":"Cinder.Theme.Behaviour.html#c:resolve_theme/0"},{"type":"module","doc":"Spark DSL for defining Cinder themes.\n\nThis module provides a DSL for creating modular, reusable theme definitions\nthat can be used across projects. It follows the same pattern as AshAuthentication's\noverride system.","title":"Cinder.Theme.Dsl","ref":"Cinder.Theme.Dsl.html"},{"type":"module","doc":"defmodule MyApp.CustomTheme do\n      use Cinder.Theme\n\n      component Cinder.Components.Table do\n        set :container_class, \"my-custom-table-container\"\n        set :row_class, \"my-custom-row hover:bg-blue-50\"\n      end\n\n      component Cinder.Components.Filters do\n        set :container_class, \"my-filter-container\"\n        set :text_input_class, \"my-text-input\"\n      end\n    end","title":"Example - Cinder.Theme.Dsl","ref":"Cinder.Theme.Dsl.html#module-example"},{"type":"function","doc":"Defines the theme DSL structure.","title":"Cinder.Theme.Dsl.dsl/0","ref":"Cinder.Theme.Dsl.html#dsl/0"},{"type":"module","doc":"Simplified DSL module for creating custom Cinder themes.\n\nThis module provides the `use Cinder.Theme` functionality that allows\nusers to define custom themes using a simple macro-based DSL.","title":"Cinder.Theme.DslModule","ref":"Cinder.Theme.DslModule.html"},{"type":"module","doc":"defmodule MyApp.CustomTheme do\n      use Cinder.Theme\n\n      component Cinder.Components.Table do\n        set :container_class, \"my-custom-table-container\"\n        set :row_class, \"my-custom-row hover:bg-blue-50\"\n      end\n\n      component Cinder.Components.Filters do\n        set :container_class, \"my-filter-container\"\n        set :text_input_class, \"my-text-input\"\n      end\n    end","title":"Usage - Cinder.Theme.DslModule","ref":"Cinder.Theme.DslModule.html#module-usage"},{"type":"module","doc":"defmodule MyApp.DarkTheme do\n      use Cinder.Theme\n      extends :modern\n\n      component Cinder.Components.Table do\n        set :container_class, \"bg-gray-900 text-white\"\n        set :row_class, \"border-gray-700 hover:bg-gray-800\"\n      end\n    end","title":"Theme Inheritance - Cinder.Theme.DslModule","ref":"Cinder.Theme.DslModule.html#module-theme-inheritance"},{"type":"macro","doc":"Before compile callback to generate the theme configuration function.","title":"Cinder.Theme.DslModule.__before_compile__/1","ref":"Cinder.Theme.DslModule.html#__before_compile__/1"},{"type":"macro","doc":"Macro for defining component customizations.","title":"Cinder.Theme.DslModule.component/2","ref":"Cinder.Theme.DslModule.html#component/2"},{"type":"macro","doc":"Macro for extending from another theme.","title":"Cinder.Theme.DslModule.extends/1","ref":"Cinder.Theme.DslModule.html#extends/1"},{"type":"function","doc":"Resolves a base theme at compile time.","title":"Cinder.Theme.DslModule.resolve_base_theme/1","ref":"Cinder.Theme.DslModule.html#resolve_base_theme/1"},{"type":"function","doc":"Resolves a theme module's DSL configuration into a theme map.","title":"Cinder.Theme.DslModule.resolve_theme/1","ref":"Cinder.Theme.DslModule.html#resolve_theme/1"},{"type":"macro","doc":"Macro for setting theme properties within an override block.","title":"Cinder.Theme.DslModule.set/2","ref":"Cinder.Theme.DslModule.html#set/2"},{"type":"function","doc":"Validates that all overrides in a theme module are valid.","title":"Cinder.Theme.DslModule.validate_theme/1","ref":"Cinder.Theme.DslModule.html#validate_theme/1"},{"type":"module","doc":"Data structure representing a theme override for a specific component.","title":"Cinder.Theme.Override","ref":"Cinder.Theme.Override.html"},{"type":"type","doc":"","title":"Cinder.Theme.Override.t/0","ref":"Cinder.Theme.Override.html#t:t/0"},{"type":"module","doc":"Data structure representing a single theme property setting.","title":"Cinder.Theme.Property","ref":"Cinder.Theme.Property.html"},{"type":"type","doc":"","title":"Cinder.Theme.Property.t/0","ref":"Cinder.Theme.Property.html#t:t/0"},{"type":"module","doc":"A compact theme with minimal spacing and high density for maximum information display.\n\nFeatures:\n- Minimal padding and margins for tight layouts\n- Smaller text sizes for information density\n- Subtle borders and clean lines\n- Efficient use of space\n- Perfect for dashboards and data-heavy interfaces","title":"Cinder.Themes.Compact","ref":"Cinder.Themes.Compact.html"},{"type":"function","doc":"Callback implementation for  Cinder.Theme.Behaviour.resolve_theme/0 .","title":"Cinder.Themes.Compact.resolve_theme/0","ref":"Cinder.Themes.Compact.html#resolve_theme/0"},{"type":"module","doc":"A DaisyUI-compatible theme following daisyUI design system.\n\nFeatures:\n- Clean, semantic class names compatible with daisyUI\n- Consistent spacing and typography\n- Table components using daisyUI table classes\n- Form inputs using daisyUI input classes\n- Button styling with daisyUI button classes","title":"Cinder.Themes.DaisyUI","ref":"Cinder.Themes.DaisyUI.html"},{"type":"function","doc":"Callback implementation for  Cinder.Theme.Behaviour.resolve_theme/0 .","title":"Cinder.Themes.DaisyUI.resolve_theme/0","ref":"Cinder.Themes.DaisyUI.html#resolve_theme/0"},{"type":"module","doc":"An elegant dark theme with smooth gradients and modern styling.\n\nFeatures:\n- Rich dark backgrounds with subtle gradients\n- Purple and blue accent colors\n- Smooth hover transitions\n- High contrast for excellent readability\n- Professional dark mode aesthetic","title":"Cinder.Themes.Dark","ref":"Cinder.Themes.Dark.html"},{"type":"function","doc":"Callback implementation for  Cinder.Theme.Behaviour.resolve_theme/0 .","title":"Cinder.Themes.Dark.resolve_theme/0","ref":"Cinder.Themes.Dark.html#resolve_theme/0"},{"type":"module","doc":"A Flowbite-compatible theme following Flowbite design system for advanced tables.\n\nFeatures:\n- Flowbite table styling with proper borders and spacing\n- Gray color scheme with hover states\n- Professional form inputs with Flowbite styling\n- Button components using Flowbite button classes\n- Consistent spacing and typography matching Flowbite docs","title":"Cinder.Themes.Flowbite","ref":"Cinder.Themes.Flowbite.html"},{"type":"function","doc":"Callback implementation for  Cinder.Theme.Behaviour.resolve_theme/0 .","title":"Cinder.Themes.Flowbite.resolve_theme/0","ref":"Cinder.Themes.Flowbite.html#resolve_theme/0"},{"type":"module","doc":"A futuristic theme with holographic effects and sci-fi styling.\n\nFeatures:\n- Dark space-like backgrounds with glowing accents\n- Holographic blue and green color scheme\n- Subtle gradients and glow effects\n- Minimalist typography with futuristic feel\n- Sharp geometric designs","title":"Cinder.Themes.Futuristic","ref":"Cinder.Themes.Futuristic.html"},{"type":"function","doc":"Callback implementation for  Cinder.Theme.Behaviour.resolve_theme/0 .","title":"Cinder.Themes.Futuristic.resolve_theme/0","ref":"Cinder.Themes.Futuristic.html#resolve_theme/0"},{"type":"module","doc":"A modern, professional theme with clean lines and subtle shadows.\n\nFeatures:\n- Clean white backgrounds with subtle shadows\n- Professional blue accent colors\n- Smooth hover transitions\n- Well-spaced typography\n- Subtle borders and rounded corners","title":"Cinder.Themes.Modern","ref":"Cinder.Themes.Modern.html"},{"type":"function","doc":"Callback implementation for  Cinder.Theme.Behaviour.resolve_theme/0 .","title":"Cinder.Themes.Modern.resolve_theme/0","ref":"Cinder.Themes.Modern.html#resolve_theme/0"},{"type":"module","doc":"A pastel theme with soft colors and gentle styling for a calm, pleasant interface.\n\nFeatures:\n- Soft pastel color palette (pinks, blues, purples, greens)\n- Gentle gradients and subtle shadows\n- Rounded corners and smooth transitions\n- Light, airy feeling with plenty of whitespace\n- Soothing design for comfortable viewing","title":"Cinder.Themes.Pastel","ref":"Cinder.Themes.Pastel.html"},{"type":"function","doc":"Callback implementation for  Cinder.Theme.Behaviour.resolve_theme/0 .","title":"Cinder.Themes.Pastel.resolve_theme/0","ref":"Cinder.Themes.Pastel.html#resolve_theme/0"},{"type":"module","doc":"A retro 80s-inspired theme with neon colors and bold styling.\n\nFeatures:\n- Dark backgrounds with neon accent colors\n- Bright cyan, magenta, and yellow highlights\n- Bold typography with strong contrast\n- Glowing effects and sharp corners\n- Classic 80s color palette","title":"Cinder.Themes.Retro","ref":"Cinder.Themes.Retro.html"},{"type":"function","doc":"Callback implementation for  Cinder.Theme.Behaviour.resolve_theme/0 .","title":"Cinder.Themes.Retro.resolve_theme/0","ref":"Cinder.Themes.Retro.html#resolve_theme/0"},{"type":"module","doc":"Simple URL synchronization helper for Table components.\n\nThis module provides an easy way to enable URL state synchronization\nfor Table components with minimal setup.","title":"Cinder.Table.UrlSync","ref":"Cinder.Table.UrlSync.html"},{"type":"module","doc":"1. Add `use Cinder.Table.UrlSync` to your LiveView\n2. Call `Cinder.Table.UrlSync.handle_params/3` in your `handle_params/3` callback\n3. Pass `url_state={@url_state}` to your Table component\n4. That's it! The helper handles all URL updates automatically.","title":"Usage - Cinder.Table.UrlSync","ref":"Cinder.Table.UrlSync.html#module-usage"},{"type":"module","doc":"defmodule MyAppWeb.UsersLive do\n      use MyAppWeb, :live_view\n      use Cinder.Table.UrlSync\n\n      def mount(_params, _session, socket) do\n        {:ok, assign(socket, :current_user, get_current_user())}\n      end\n\n      def handle_params(params, uri, socket) do\n        socket = Cinder.Table.UrlSync.handle_params(params, uri, socket)\n        {:noreply, socket}\n      end\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n          <:col field=\"name\" filter sort>Name</:col>\n          <:col field=\"email\" filter>Email</:col>\n         \n        \"\"\"\n      end\n\n      # Or with a pre-configured query:\n      def render_with_query(assigns) do\n        ~H\"\"\"\n          Ash.Query.filter(active: true)}\n          actor={@current_user}\n          url_state={@url_state}\n        >\n          <:col field=\"name\" filter sort>Name</:col>\n          <:col field=\"email\" filter>Email</:col>\n         \n        \"\"\"\n      end\n    end\n\nThe helper automatically:\n- Handles `:table_state_change` messages from Table components\n- Updates the URL with new table state\n- Preserves other URL parameters\n- Works with any number of Table components on the same page","title":"Example - Cinder.Table.UrlSync","ref":"Cinder.Table.UrlSync.html#module-example"},{"type":"macro","doc":"Adds URL sync support to a LiveView.\n\nThis macro injects the necessary `handle_info/2` callback to handle\ntable state changes and update the URL accordingly.","title":"Cinder.Table.UrlSync.__using__/1","ref":"Cinder.Table.UrlSync.html#__using__/1"},{"type":"function","doc":"Extracts table state from URL parameters using empty columns list.\n\nThis function provides a simplified extraction that works without column\nmetadata. It preserves page and sort information but may not fully decode\nfilters (which is why we also preserve raw params in handle_params).","title":"Cinder.Table.UrlSync.extract_table_state/1","ref":"Cinder.Table.UrlSync.html#extract_table_state/1"},{"type":"function","doc":"- `params` - URL parameters map","title":"Parameters - Cinder.Table.UrlSync.extract_table_state/1","ref":"Cinder.Table.UrlSync.html#extract_table_state/1-parameters"},{"type":"function","doc":"Map with `:filters`, `:current_page`, and `:sort_by` keys","title":"Returns - Cinder.Table.UrlSync.extract_table_state/1","ref":"Cinder.Table.UrlSync.html#extract_table_state/1-returns"},{"type":"function","doc":"Extracts complete URL state from URL parameters for use with table components.\n\nThis function can be used in `handle_params/3` to initialize\nURL state for table components.","title":"Cinder.Table.UrlSync.extract_url_state/1","ref":"Cinder.Table.UrlSync.html#extract_url_state/1"},{"type":"function","doc":"def handle_params(params, _uri, socket) do\n      url_state = Cinder.Table.UrlSync.extract_url_state(params)\n      socket = assign(socket, :url_state, url_state)\n      {:noreply, socket}\n    end","title":"Example - Cinder.Table.UrlSync.extract_url_state/1","ref":"Cinder.Table.UrlSync.html#extract_url_state/1-example"},{"type":"function","doc":"Helper to get the URL state for passing to table components.\n\nUse this to get the URL state object created by `handle_params/3`.","title":"Cinder.Table.UrlSync.get_url_state/1","ref":"Cinder.Table.UrlSync.html#get_url_state/1"},{"type":"function","doc":"def render(assigns) do\n      ~H\"\"\"\n       \n        <:col field=\"name\" filter=\"text\">Name</:col>\n        <:col field=\"artist.name\" filter=\"text\">Artist</:col>\n       \n      \"\"\"\n    end\n\nThe URL state object contains:\n- filters: Raw URL parameters for proper filter decoding\n- current_page: Current page number\n- sort_by: Sort configuration\n- uri: Current URI","title":"Example - Cinder.Table.UrlSync.get_url_state/1","ref":"Cinder.Table.UrlSync.html#get_url_state/1-example"},{"type":"function","doc":"Helper function to handle table state in LiveView handle_params.\n\nThis function extracts URL parameters and creates a URL state object that\ncan be passed to Table components. It should be called from your LiveView's\n`handle_params/3` callback.","title":"Cinder.Table.UrlSync.handle_params/3","ref":"Cinder.Table.UrlSync.html#handle_params/3"},{"type":"function","doc":"- `params` - URL parameters from `handle_params/3`\n- `uri` - Current URI from `handle_params/3` (optional but recommended)\n- `socket` - The LiveView socket","title":"Parameters - Cinder.Table.UrlSync.handle_params/3","ref":"Cinder.Table.UrlSync.html#handle_params/3-parameters"},{"type":"function","doc":"Updated socket with `:url_state` assign containing:\n- `filters` - Raw URL parameters for proper filter decoding\n- `current_page` - Current page number\n- `sort_by` - Sort configuration\n- `uri` - Current URI for URL generation","title":"Returns - Cinder.Table.UrlSync.handle_params/3","ref":"Cinder.Table.UrlSync.html#handle_params/3-returns"},{"type":"function","doc":"def handle_params(params, uri, socket) do\n      socket = Cinder.Table.UrlSync.handle_params(params, uri, socket)\n      {:noreply, socket}\n    end\n\n    def render(assigns) do\n      ~H\"\"\"\n       \n        <:col field=\"name\" filter sort>Name</:col>\n       \n      \"\"\"\n    end\n\nThe `@url_state` assign will be available for use with the Table component.","title":"Example - Cinder.Table.UrlSync.handle_params/3","ref":"Cinder.Table.UrlSync.html#handle_params/3-example"},{"type":"function","doc":"Updates the LiveView socket with new URL parameters.\n\nThis function preserves the current path and updates only the query parameters\nwith the new table state.","title":"Cinder.Table.UrlSync.update_url/3","ref":"Cinder.Table.UrlSync.html#update_url/3"},{"type":"function","doc":"- `socket` - The LiveView socket\n- `encoded_state` - Map of URL parameters from table state\n- `current_uri` - Optional current URI string to use for path resolution","title":"Parameters - Cinder.Table.UrlSync.update_url/3","ref":"Cinder.Table.UrlSync.html#update_url/3-parameters"},{"type":"function","doc":"Updated socket with URL changed via `push_patch/2`","title":"Returns - Cinder.Table.UrlSync.update_url/3","ref":"Cinder.Table.UrlSync.html#update_url/3-returns"},{"type":"module","doc":"URL state management for Cinder table components.\n\nHandles encoding and decoding of table state (filters, pagination, sorting)\nto/from URL parameters for browser history and bookmark support.","title":"Cinder.UrlManager","ref":"Cinder.UrlManager.html"},{"type":"function","doc":"Decodes filters from URL parameters using column definitions.\n\nUses column metadata to properly parse filter values according to their types.","title":"Cinder.UrlManager.decode_filters/2","ref":"Cinder.UrlManager.html#decode_filters/2"},{"type":"function","doc":"Decodes page number from URL parameter.\n\nReturns 1 for invalid or missing page parameters.","title":"Cinder.UrlManager.decode_page/1","ref":"Cinder.UrlManager.html#decode_page/1"},{"type":"function","doc":"iex> Cinder.UrlManager.decode_page(\"5\")\n    5\n\n    iex> Cinder.UrlManager.decode_page(\"invalid\")\n    1\n\n    iex> Cinder.UrlManager.decode_page(nil)\n    1","title":"Examples - Cinder.UrlManager.decode_page/1","ref":"Cinder.UrlManager.html#decode_page/1-examples"},{"type":"function","doc":"Decodes sort string from URL parameters.\n\nParses Ash sort string format into sort tuples.\nFields prefixed with \"-\" are descending, others are ascending.","title":"Cinder.UrlManager.decode_sort/1","ref":"Cinder.UrlManager.html#decode_sort/1"},{"type":"function","doc":"iex> Cinder.UrlManager.decode_sort(\"-title,created_at\")\n    [{\"title\", :desc}, {\"created_at\", :asc}]","title":"Examples - Cinder.UrlManager.decode_sort/1","ref":"Cinder.UrlManager.html#decode_sort/1-examples"},{"type":"function","doc":"Decodes URL parameters into table state components.\n\nTakes URL parameters and column definitions to properly decode filter values\nbased on their types.","title":"Cinder.UrlManager.decode_state/2","ref":"Cinder.UrlManager.html#decode_state/2"},{"type":"function","doc":"iex> url_params = %{\"title\" => \"test\", \"page\" => \"2\", \"sort\" => \"-title\"}\n    iex> columns = [%{field: \"title\", filterable: true, filter_type: :text}]\n    iex> Cinder.UrlManager.decode_state(url_params, columns)\n    %{\n      filters: %{\"title\" => %{type: :text, value: \"test\", operator: :contains}},\n      current_page: 2,\n      sort_by: [{\"title\", :desc}]\n    }","title":"Examples - Cinder.UrlManager.decode_state/2","ref":"Cinder.UrlManager.html#decode_state/2-examples"},{"type":"function","doc":"Encodes filters for URL parameters.\n\nConverts filter values to strings appropriate for URL encoding.\nDifferent filter types are encoded differently:\n- Multi-select: comma-separated values\n- Date/number ranges: \"from,to\" or \"min,max\" format\n- Others: string representation","title":"Cinder.UrlManager.encode_filters/1","ref":"Cinder.UrlManager.html#encode_filters/1"},{"type":"function","doc":"Encodes sort state for URL parameters.\n\nConverts sort tuples to Ash-compatible sort string format.\nDescending sorts are prefixed with \"-\".","title":"Cinder.UrlManager.encode_sort/1","ref":"Cinder.UrlManager.html#encode_sort/1"},{"type":"function","doc":"iex> Cinder.UrlManager.encode_sort([{\"title\", :desc}, {\"created_at\", :asc}])\n    \"-title,created_at\"","title":"Examples - Cinder.UrlManager.encode_sort/1","ref":"Cinder.UrlManager.html#encode_sort/1-examples"},{"type":"function","doc":"Encodes table state into URL parameters.","title":"Cinder.UrlManager.encode_state/1","ref":"Cinder.UrlManager.html#encode_state/1"},{"type":"function","doc":"iex> state = %{\n    ...>   filters: %{\"title\" => %{type: :text, value: \"test\", operator: :contains}},\n    ...>   current_page: 2,\n    ...>   sort_by: [{\"title\", :desc}]\n    ...> }\n    iex> Cinder.UrlManager.encode_state(state)\n    %{title: \"test\", page: \"2\", sort: \"-title\"}","title":"Examples - Cinder.UrlManager.encode_state/1","ref":"Cinder.UrlManager.html#encode_state/1-examples"},{"type":"function","doc":"Ensures multi-select fields are included in filter parameters.\n\nMulti-select filters that have no selected values need to be explicitly\nincluded as empty arrays to distinguish from filters that weren't processed.","title":"Cinder.UrlManager.ensure_multiselect_fields/2","ref":"Cinder.UrlManager.html#ensure_multiselect_fields/2"},{"type":"type","doc":"","title":"Cinder.UrlManager.filter/0","ref":"Cinder.UrlManager.html#t:filter/0"},{"type":"type","doc":"","title":"Cinder.UrlManager.filter_value/0","ref":"Cinder.UrlManager.html#t:filter_value/0"},{"type":"function","doc":"Sends state change notification to parent LiveView.\n\nUsed by components to notify their parent when table state changes,\nallowing the parent to update the URL accordingly.","title":"Cinder.UrlManager.notify_state_change/2","ref":"Cinder.UrlManager.html#notify_state_change/2"},{"type":"type","doc":"","title":"Cinder.UrlManager.sort_by/0","ref":"Cinder.UrlManager.html#t:sort_by/0"},{"type":"type","doc":"","title":"Cinder.UrlManager.table_state/0","ref":"Cinder.UrlManager.html#t:table_state/0"},{"type":"type","doc":"","title":"Cinder.UrlManager.url_params/0","ref":"Cinder.UrlManager.html#t:url_params/0"},{"type":"function","doc":"Validates URL parameters for potential security issues.\n\nPerforms basic validation to ensure URL parameters are safe to process.\nReturns {:ok, params} for valid parameters or {:error, reason} for invalid ones.","title":"Cinder.UrlManager.validate_url_params/1","ref":"Cinder.UrlManager.html#validate_url_params/1"},{"type":"module","doc":"Column configuration and type inference for Cinder table components.\n\nProvides intelligent column parsing that can automatically infer filter types,\nsort capabilities, and display options from Ash resource definitions.\nSupports relationship fields using dot notation (e.g., \"user.name\").","title":"Cinder.Column","ref":"Cinder.Column.html"},{"type":"function","doc":"Infers column configuration from Ash resource attribute definitions.","title":"Cinder.Column.infer_from_resource/4","ref":"Cinder.Column.html#infer_from_resource/4"},{"type":"function","doc":"Merges slot configuration with inferred defaults.","title":"Cinder.Column.merge_config/2","ref":"Cinder.Column.html#merge_config/2"},{"type":"function","doc":"Parses a single column definition with automatic type inference.","title":"Cinder.Column.parse_column/2","ref":"Cinder.Column.html#parse_column/2"},{"type":"function","doc":"Parses and normalizes column definitions from slots and resource information.","title":"Cinder.Column.parse_columns/2","ref":"Cinder.Column.html#parse_columns/2"},{"type":"function","doc":"- `slots` - List of column slot definitions\n- `resource` - Ash resource module for type inference","title":"Parameters - Cinder.Column.parse_columns/2","ref":"Cinder.Column.html#parse_columns/2-parameters"},{"type":"function","doc":"List of normalized Column structs","title":"Returns - Cinder.Column.parse_columns/2","ref":"Cinder.Column.html#parse_columns/2-returns"},{"type":"type","doc":"","title":"Cinder.Column.t/0","ref":"Cinder.Column.html#t:t/0"},{"type":"function","doc":"Validates a column configuration.","title":"Cinder.Column.validate/1","ref":"Cinder.Column.html#validate/1"},{"type":"module","doc":"Coordinator for Cinder's modular filter system.\n\nActs as the main interface for filter operations while delegating\nto individual filter type modules for specific implementations.","title":"Cinder.FilterManager","ref":"Cinder.FilterManager.html"},{"type":"function","doc":"Builds filter values map for form inputs using modular filter system.","title":"Cinder.FilterManager.build_filter_values/2","ref":"Cinder.FilterManager.html#build_filter_values/2"},{"type":"function","doc":"Clears all filters from the filter state.","title":"Cinder.FilterManager.clear_all_filters/1","ref":"Cinder.FilterManager.html#clear_all_filters/1"},{"type":"function","doc":"Clears a specific filter from the filter state.","title":"Cinder.FilterManager.clear_filter/2","ref":"Cinder.FilterManager.html#clear_filter/2"},{"type":"type","doc":"","title":"Cinder.FilterManager.column/0","ref":"Cinder.FilterManager.html#t:column/0"},{"type":"function","doc":"Counts the number of active filters.","title":"Cinder.FilterManager.count_active_filters/1","ref":"Cinder.FilterManager.html#count_active_filters/1"},{"type":"type","doc":"","title":"Cinder.FilterManager.filter/0","ref":"Cinder.FilterManager.html#t:filter/0"},{"type":"function","doc":"Renders an individual filter input by delegating to the appropriate filter module.","title":"Cinder.FilterManager.filter_input/1","ref":"Cinder.FilterManager.html#filter_input/1"},{"type":"type","doc":"","title":"Cinder.FilterManager.filter_type/0","ref":"Cinder.FilterManager.html#t:filter_type/0"},{"type":"type","doc":"","title":"Cinder.FilterManager.filter_value/0","ref":"Cinder.FilterManager.html#t:filter_value/0"},{"type":"type","doc":"","title":"Cinder.FilterManager.filters/0","ref":"Cinder.FilterManager.html#t:filters/0"},{"type":"function","doc":"Checks if a filter has a meaningful value using modular system.","title":"Cinder.FilterManager.has_filter_value?/1","ref":"Cinder.FilterManager.html#has_filter_value?/1"},{"type":"function","doc":"Infers filter configuration from Ash resource attribute definitions.","title":"Cinder.FilterManager.infer_filter_config/3","ref":"Cinder.FilterManager.html#infer_filter_config/3"},{"type":"function","doc":"Transforms form values into structured filter objects using modular filter system.","title":"Cinder.FilterManager.params_to_filters/2","ref":"Cinder.FilterManager.html#params_to_filters/2"},{"type":"function","doc":"Processes filter parameters from form submission using modular filter system.","title":"Cinder.FilterManager.process_filter_params/2","ref":"Cinder.FilterManager.html#process_filter_params/2"},{"type":"function","doc":"Processes raw filter value using the appropriate filter module.\n\nThis function is public to enable comprehensive testing of custom filter processing.","title":"Cinder.FilterManager.process_filter_value/2","ref":"Cinder.FilterManager.html#process_filter_value/2"},{"type":"function","doc":"Renders filter controls for a list of columns.","title":"Cinder.FilterManager.render_filter_controls/1","ref":"Cinder.FilterManager.html#render_filter_controls/1"},{"type":"function","doc":"- `columns` - List of column definitions\n- `filters` - Current filter state map\n- `theme` - Theme configuration\n- `target` - LiveComponent target for events","title":"Parameters - Cinder.FilterManager.render_filter_controls/1","ref":"Cinder.FilterManager.html#render_filter_controls/1-parameters"},{"type":"function","doc":"HEEx template for filter controls","title":"Returns - Cinder.FilterManager.render_filter_controls/1","ref":"Cinder.FilterManager.html#render_filter_controls/1-returns"},{"type":"function","doc":"Validates all registered custom filters at application startup.\n\nThis function should be called during application initialization to ensure\nall custom filters are properly implemented and available.","title":"Cinder.FilterManager.validate_runtime_filters/0","ref":"Cinder.FilterManager.html#validate_runtime_filters/0"},{"type":"function","doc":":ok if all filters are valid, logs warnings for any issues","title":"Returns - Cinder.FilterManager.validate_runtime_filters/0","ref":"Cinder.FilterManager.html#validate_runtime_filters/0-returns"},{"type":"function","doc":"# In your application.ex start/2 function\n    case Cinder.FilterManager.validate_runtime_filters() do\n      :ok -> :ok\n      {:error, _} -> :ok  # Continue startup but log issues\n    end","title":"Examples - Cinder.FilterManager.validate_runtime_filters/0","ref":"Cinder.FilterManager.html#validate_runtime_filters/0-examples"},{"type":"module","doc":"Query building functionality for Cinder table components.\n\nHandles the construction of Ash queries with filters, sorting, and pagination\nfor table data loading.","title":"Cinder.QueryBuilder","ref":"Cinder.QueryBuilder.html"},{"type":"function","doc":"Applies filters to an Ash query based on filter configuration and column definitions.","title":"Cinder.QueryBuilder.apply_filters/3","ref":"Cinder.QueryBuilder.html#apply_filters/3"},{"type":"function","doc":"Applies query options like load and select to an Ash query.","title":"Cinder.QueryBuilder.apply_query_opts/2","ref":"Cinder.QueryBuilder.html#apply_query_opts/2"},{"type":"function","doc":"Applies sorting to an Ash query based on sort specifications and column definitions.","title":"Cinder.QueryBuilder.apply_sorting/3","ref":"Cinder.QueryBuilder.html#apply_sorting/3"},{"type":"function","doc":"Applies standard filters by delegating to the appropriate filter module.","title":"Cinder.QueryBuilder.apply_standard_filter/4","ref":"Cinder.QueryBuilder.html#apply_standard_filter/4"},{"type":"function","doc":"Builds a complete query with filters, sorting, and pagination.","title":"Cinder.QueryBuilder.build_and_execute/2","ref":"Cinder.QueryBuilder.html#build_and_execute/2"},{"type":"function","doc":"- `resource`: The Ash resource to query\n- `options`: Query building options including:\n  - `:actor` - The current user/actor\n  - `:filters` - Filter map\n  - `:sort_by` - Sort specifications\n  - `:page_size` - Number of records per page\n  - `:current_page` - Current page number\n  - `:columns` - Column definitions\n  - `:query_opts` - Additional Ash query and execution options","title":"Parameters - Cinder.QueryBuilder.build_and_execute/2","ref":"Cinder.QueryBuilder.html#build_and_execute/2-parameters"},{"type":"function","doc":"The `:query_opts` parameter accepts both query building and execution options:","title":"Supported Query Options - Cinder.QueryBuilder.build_and_execute/2","ref":"Cinder.QueryBuilder.html#build_and_execute/2-supported-query-options"},{"type":"function","doc":"- `:select` - Select specific attributes (handled by `Ash.Query.select/2`)\n- `:load` - Load relationships and calculations (handled by `Ash.Query.load/2`)","title":"Query Building Options - Cinder.QueryBuilder.build_and_execute/2","ref":"Cinder.QueryBuilder.html#build_and_execute/2-query-building-options"},{"type":"function","doc":"These options are passed to both `Ash.Query.for_read/3` and `Ash.read/2`:\n- `:timeout` - Query timeout in milliseconds or `:infinity` (e.g., `:timer.seconds(30)`)\n- `:authorize?` - Whether to run authorization during query execution\n- `:max_concurrency` - Maximum number of processes for parallel loading","title":"Execution Options - Cinder.QueryBuilder.build_and_execute/2","ref":"Cinder.QueryBuilder.html#build_and_execute/2-execution-options"},{"type":"function","doc":"# Simple timeout for long-running queries\n    query_opts: [timeout: :timer.seconds(30)]\n\n    # Query building options\n    query_opts: [select: [:name, :email], load: [:posts]]\n\n    # Combined query building and execution options\n    query_opts: [\n      timeout: :timer.seconds(20),\n      authorize?: false,\n      select: [:title, :content],\n      load: [:author, :comments]\n    ]","title":"Usage Examples - Cinder.QueryBuilder.build_and_execute/2","ref":"Cinder.QueryBuilder.html#build_and_execute/2-usage-examples"},{"type":"function","doc":"A tuple `{:ok, {results, page_info}}` or `{:error, reason}`","title":"Returns - Cinder.QueryBuilder.build_and_execute/2","ref":"Cinder.QueryBuilder.html#build_and_execute/2-returns"},{"type":"function","doc":"Builds error pagination info for failed queries.","title":"Cinder.QueryBuilder.build_error_page_info/0","ref":"Cinder.QueryBuilder.html#build_error_page_info/0"},{"type":"function","doc":"Builds expression sort for relationship fields using dot notation.","title":"Cinder.QueryBuilder.build_expression_sort/1","ref":"Cinder.QueryBuilder.html#build_expression_sort/1"},{"type":"function","doc":"Builds pagination info from query results and total count.","title":"Cinder.QueryBuilder.build_page_info_with_total_count/4","ref":"Cinder.QueryBuilder.html#build_page_info_with_total_count/4"},{"type":"type","doc":"","title":"Cinder.QueryBuilder.column/0","ref":"Cinder.QueryBuilder.html#t:column/0"},{"type":"type","doc":"","title":"Cinder.QueryBuilder.filter/0","ref":"Cinder.QueryBuilder.html#t:filter/0"},{"type":"type","doc":"","title":"Cinder.QueryBuilder.filters/0","ref":"Cinder.QueryBuilder.html#t:filters/0"},{"type":"function","doc":"Gets the current sort direction for a given key.","title":"Cinder.QueryBuilder.get_sort_direction/2","ref":"Cinder.QueryBuilder.html#get_sort_direction/2"},{"type":"type","doc":"","title":"Cinder.QueryBuilder.query_opts/0","ref":"Cinder.QueryBuilder.html#t:query_opts/0"},{"type":"type","doc":"","title":"Cinder.QueryBuilder.sort_by/0","ref":"Cinder.QueryBuilder.html#t:sort_by/0"},{"type":"function","doc":"Toggles sort direction for a given key in the sort specification.\n\nCycles through: none → ascending → descending → none","title":"Cinder.QueryBuilder.toggle_sort_direction/2","ref":"Cinder.QueryBuilder.html#toggle_sort_direction/2"},{"type":"extras","doc":"# Cinder\n\nA powerful, intelligent data table component for Ash Framework resources, in your Phoenix LiveView applications.","title":"Cinder","ref":"readme.html"},{"type":"extras","doc":"Cinder transforms complex data table requirements into simple, declarative markup. With automatic type inference and intelligent defaults, you can build feature-rich tables for Ash resources and queries, with minimal configuration.\n\n```elixir\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" filter>{user.email}</:col>\n  <:col :let={user} field=\"department.name\" filter sort>{user.department.name}</:col>\n \n```\n\nThat's it! Cinder automatically provides:\n- ✅ Intelligent filter types based on your Ash resource\n- ✅ Interactive sorting with visual indicators\n- ✅ Pagination with efficient queries\n- ✅ Relationship support via dot notation\n- ✅ URL state management for bookmarkable views\n\n \n   \n   \n \n\n*Sort and filter by calculations, aggregates, attributes, or even relationship data!*","title":"What is Cinder? - Cinder","ref":"readme.html#what-is-cinder"},{"type":"extras","doc":"- **🧠 Intelligent Defaults**: Automatic filter type detection from Ash resource attributes\n- **⚡ Minimal Configuration**: 70% fewer attributes required compared to traditional table components\n- **🔗 Complete URL State Management**: Filters, pagination, and sorting synchronized with browser URL\n- **🌐 Relationship Support**: Dot notation for related fields (e.g., `user.department.name`)\n- **🖱️ Interactive Row Actions**: Click handlers with Phoenix LiveView JS commands for navigation, modals, and custom actions\n- **🎨 Advanced Theming**: 8 built-in themes (modern, retro, futuristic, dark, daisy_ui, flowbite, compact, pastel) plus powerful DSL for custom themes\n- **🔧 Developer Experience**: Data attributes on every element make theme development and debugging effortless\n- **⚡ Real-time Filtering**: Six filter types with debounced updates\n- **🔐 Ash Integration**: Native support for Ash Framework resources and authorization","title":"Key Features - Cinder","ref":"readme.html#key-features"},{"type":"extras","doc":"","title":"Installation - Cinder","ref":"readme.html#installation"},{"type":"extras","doc":"If you're using [Igniter](https://hexdocs.pm/igniter) in your project:\n\n```bash\nmix igniter.install cinder\n```\n\nThis will automatically:\n- Add Cinder to your dependencies\n- Configure Tailwind CSS to include Cinder's styles\n- Provide setup instructions and examples","title":"Using Igniter (Recommended) - Cinder","ref":"readme.html#using-igniter-recommended"},{"type":"extras","doc":"Add `cinder` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:cinder, \"~> 0.2\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\nmix cinder.install  # Configure Tailwind CSS\n```\n\nThe installer will automatically update your Tailwind configuration to include Cinder's CSS classes. If automatic configuration fails, it will provide manual setup instructions.","title":"Manual Installation - Cinder","ref":"readme.html#manual-installation"},{"type":"extras","doc":"","title":"Quick Start - Cinder","ref":"readme.html#quick-start"},{"type":"extras","doc":"```elixir\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" filter>{user.email}</:col>\n  <:col :let={user} field=\"created_at\" sort>{user.created_at}</:col>\n \n```","title":"Basic Table - Cinder","ref":"readme.html#basic-table"},{"type":"extras","doc":"For complex requirements, use the `query` parameter:\n\n```elixir\n  Ash.Query.filter(active: true)} actor={@current_user}>\n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" filter>{user.email}</:col>\n \n```","title":"Advanced Query Usage - Cinder","ref":"readme.html#advanced-query-usage"},{"type":"extras","doc":"#### Row Click Actions\n\nMake your tables interactive by adding click handlers to rows. The `row_click` attribute accepts a function that receives the row item and returns a Phoenix LiveView JS command:\n\n```elixir\n  JS.navigate(~p\"/users/#{user.id}\") end}\n>\n  <:col field=\"name\" filter sort>Name</:col>\n  <:col field=\"email\" filter>Email</:col>\n  <:col field=\"role\" filter>Role</:col>\n \n```\n\nWhen `row_click` is provided:\n- Rows become visually clickable with cursor pointer styling\n- Clicking anywhere on a row executes your function\n- Works with any Phoenix LiveView JS command (navigate, show modal, dispatch events, etc.)\n\nExample uses:\n- Navigate to detail pages\n- Open modals or slideovers\n- Trigger custom events\n- Show/hide content\n- Execute any client-side JavaScript\n\n```elixir\n# Navigate to user detail page\nrow_click={fn user -> JS.navigate(~p\"/users/#{user.id}\") end}\n\n# Show a modal\nrow_click={fn user -> JS.show(to: \"#user-modal-#{user.id}\") end}\n\n# Dispatch a custom event\nrow_click={fn user -> JS.dispatch(\"user:selected\", detail: %{id: user.id}) end}\n\n# Chain multiple actions\nrow_click={fn user -> \n  JS.navigate(~p\"/users/#{user.id}\")\n  |> JS.push(\"track_click\", value: %{user_id: user.id})\nend}\n```","title":"Interactive Features - Cinder","ref":"readme.html#interactive-features"},{"type":"extras","doc":"You can configure a default theme for all Cinder tables in your application:\n\n```elixir\n# config/config.exs\nconfig :cinder, default_theme: \"modern\"\n```\n\nThis theme will be used by all tables unless explicitly overridden:\n\n```elixir\n<!-- Uses configured default theme -->\n \n  <:col field=\"name\" filter sort>Name</:col>\n \n\n<!-- Overrides default with specific theme -->\n \n  <:col field=\"name\" filter sort>Name</:col>\n \n```\n\nYou can also use custom theme modules:\n\n```elixir\n# config/config.exs\nconfig :cinder, default_theme: MyApp.CustomTheme\n```\n\nAvailable built-in themes: `\"default\"`, `\"modern\"`, `\"retro\"`, `\"futuristic\"`, `\"dark\"`, `\"daisy_ui\"`, `\"flowbite\"`, `\"compact\"`, `\"pastel\"`","title":"Default Theme Configuration - Cinder","ref":"readme.html#default-theme-configuration"},{"type":"extras","doc":"Cinder can automatically update the URL as you filter and sort your tables, for bookmarkable and shareable URLs.\n\n1. Add `use Cinder.Table.UrlSync` to your LiveView modules\n\n2. Add an `handle_params` callback function (or update your existing one) to add a call to `Cinder.Table.UrlSync.handle_params`. This will read the current table state from the URL, and store it in a new `url_state` assign.\n\n3. Use the `url_state` option when defining your table, eg. `Cinder.Table.table url_state={@url_state}`.\n\nAnd that's it! Everything else is handled for you.\n\n```elixir\ndefmodule MyAppWeb.UsersLive do\n  use MyAppWeb, :live_view\n  use Cinder.Table.UrlSync\n\n  def handle_params(params, uri, socket) do\n    socket = Cinder.Table.UrlSync.handle_params(params, uri, socket)\n    {:noreply, socket}\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n      <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n     \n    \"\"\"\n  end\nend\n```","title":"URL State Management - Cinder","ref":"readme.html#url-state-management"},{"type":"extras","doc":"- **`Cinder.Table`** - All the configuration options for `table` components and `col` slots.\n- **[Complete Examples](docs/examples.md)** - Comprehensive usage examples for all features\n- **[Theming Guide](docs/theming.md)** - How to develop and use table themes\n- **[Module Documentation](https://hexdocs.pm/cinder)** - Full API reference\n- **[Hex Package](https://hex.pm/packages/cinder)** - Package information\n\nFor detailed examples of filters, sorting, theming, relationships, and advanced query usage, see the [examples documentation](docs/examples.md).","title":"Documentation - Cinder","ref":"readme.html#documentation"},{"type":"extras","doc":"- Phoenix LiveView 1.0+\n- Ash Framework 3.0+\n- Elixir 1.17+","title":"Requirements - Cinder","ref":"readme.html#requirements"},{"type":"extras","doc":"Contributions are welcome! Please submit pull requests to our GitHub repository.","title":"Contributing - Cinder","ref":"readme.html#contributing"},{"type":"extras","doc":"This project is licensed under the MIT License - see the LICENSE file for details.","title":"License - Cinder","ref":"readme.html#license"},{"type":"extras","doc":"# Examples\n\nThis document provides comprehensive examples and detailed reference for all Cinder table features. For a quick start, see the [README](../README.md).","title":"Examples","ref":"examples.html"},{"type":"extras","doc":"Cinder supports two parameter styles:\n- **`resource`** - Simple usage with Ash resource modules\n- **`query`** - Advanced usage with pre-configured Ash queries\n\nChoose `resource` for most cases, `query` for complex requirements like custom read actions, base filters, or admin interfaces.","title":"Overview - Examples","ref":"examples.html#overview"},{"type":"extras","doc":"- [Basic Usage](#basic-usage)\n- [Resource vs Query](#resource-vs-query)\n- [Column Configuration](#column-configuration)\n- [Filter Types](#filter-types)\n- [Sorting](#sorting)\n- [Theming](#theming)\n- [URL State Management](#url-state-management)\n- [Relationship Fields](#relationship-fields)\n- [Custom Content](#custom-content)\n- [Advanced Configuration](#advanced-configuration)\n- [Performance Optimization](#performance-optimization)","title":"Table of Contents - Examples","ref":"examples.html#table-of-contents"},{"type":"extras","doc":"","title":"Basic Usage - Examples","ref":"examples.html#basic-usage"},{"type":"extras","doc":"The simplest possible table:\n\n```elixir\n \n  <:col :let={user} field=\"name\">{user.name}</:col>\n  <:col :let={user} field=\"email\">{user.email}</:col>\n \n```","title":"Minimal Table - Examples","ref":"examples.html#minimal-table"},{"type":"extras","doc":"Cinder supports two ways to specify what data to query: `resource` parameter (simple) or `query` parameter (advanced).","title":"Resource vs Query - Examples","ref":"examples.html#resource-vs-query"},{"type":"extras","doc":"Use the `resource` parameter for straightforward tables:\n\n```elixir\n<!-- Simple table with default read action -->\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" filter>{user.email}</:col>\n \n```\n\n**Best for:**\n- Getting started quickly\n- Standard use cases without custom requirements\n- Default read actions\n- Simple authorization scenarios","title":"When to Use Resource - Examples","ref":"examples.html#when-to-use-resource"},{"type":"extras","doc":"Use the `query` parameter for advanced scenarios:\n\n```elixir\n<!-- Custom read action -->\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" filter>{user.email}</:col>\n \n\n<!-- Pre-filtered data -->\n  Ash.Query.filter(department: \"Engineering\")} actor={@current_user}>\n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"department.name\" filter>{user.department.name}</:col>\n \n\n<!-- Admin interface with complex authorization -->\n  Ash.Query.for_read(:admin_read, %{}, actor: @actor, authorize?: @authorizing)\n    |> Ash.Query.set_tenant(@tenant)\n    |> Ash.Query.filter(active: true)}\n  actor={@current_user}>\n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" filter>{user.email}</:col>\n  <:col :let={user} field=\"last_login\" sort>{user.last_login}</:col>\n \n```\n\n**Best for:**\n- Custom read actions (e.g., `:active_users`, `:admin_only`)\n- Pre-filtering data with base filters\n- Custom authorization settings\n- Tenant-specific queries\n- Admin interfaces with complex requirements\n- Integration with existing Ash query pipelines","title":"When to Use Query - Examples","ref":"examples.html#when-to-use-query"},{"type":"extras","doc":"Cinder automatically generates human-readable labels from field names:\n\n```elixir\n \n  <:col :let={user} field=\"first_name\">{user.first_name}</:col>      <!-- \"First Name\" -->\n  <:col :let={user} field=\"email_address\">{user.email_address}</:col>   <!-- \"Email Address\" -->\n  <:col :let={user} field=\"created_at\">{user.created_at}</:col>      <!-- \"Created At\" -->\n  <:col :let={user} field=\"is_active\">{user.is_active}</:col>       <!-- \"Is Active\" -->\n  <:col :let={user} field=\"phone_number\">{user.phone_number}</:col>    <!-- \"Phone Number\" -->\n \n```","title":"Automatic Label Generation - Examples","ref":"examples.html#automatic-label-generation"},{"type":"extras","doc":"Override auto-generated labels when needed:\n\n```elixir\n \n  <:col :let={user} field=\"name\" label=\"Full Name\">{user.name}</:col>\n  <:col :let={user} field=\"email\" label=\"Email Address\">{user.email}</:col>\n  <:col :let={user} field=\"created_at\" label=\"Joined\">{user.created_at}</:col>\n  <:col :let={user} field=\"is_active\" label=\"Status\">{user.is_active}</:col>\n \n```","title":"Custom Labels - Examples","ref":"examples.html#custom-labels"},{"type":"extras","doc":"","title":"Column Configuration - Examples","ref":"examples.html#column-configuration"},{"type":"extras","doc":"Demonstration of every available column attribute:\n\n```elixir\n \n  <!-- Basic column with all common attributes -->\n  <:col\n    :let={product}\n    field=\"name\"\n    label=\"Product Name\"\n    filter={:text}\n    sort={true}\n    class=\"w-1/4 font-semibold\"\n  >\n    {product.name}\n  </:col>\n\n  <!-- Column with custom filter options -->\n  <:col\n    :let={product}\n    field=\"category\"\n    filter={:select}\n    filter_options={[\n      options: [{\"Electronics\", \"electronics\"}, {\"Books\", \"books\"}, {\"Clothing\", \"clothing\"}],\n      prompt: \"All Categories\"\n    ]}\n    sort\n  >\n    {product.category}\n  </:col>\n\n  <!-- Number column with range filter -->\n  <:col\n    :let={product}\n    field=\"price\"\n    filter={:number_range}\n    filter_options={[\n      min: 0,\n      max: 1000,\n      step: 0.01\n    ]}\n    sort\n    class=\"text-right\"\n  >\n    ${product.price}\n  </:col>\n\n  <!-- Boolean column with custom labels -->\n  <:col\n    :let={product}\n    field=\"in_stock\"\n    filter={:boolean}\n    filter_options={[\n      labels: %{\n        all: \"Any Stock Status\",\n        true: \"In Stock\",\n        false: \"Out of Stock\"\n      }\n    ]}\n  >\n    {if product.in_stock, do: \"In Stock\", else: \"Out of Stock\"}\n  </:col>\n \n```","title":"All Column Attributes - Examples","ref":"examples.html#all-column-attributes"},{"type":"extras","doc":"Cinder automatically detects the right filter type based on your Ash resource attributes:\n\n- **String fields** → Text search\n- **Enum fields** → Select dropdown\n- **Boolean fields** → True/false/any radio buttons\n- **Date/DateTime fields** → Date range picker\n- **Integer/Decimal fields** → Number range inputs\n- **Array fields** → Multi-select tag interface\n\nYou can also explicitly specify filter types: `:text`, `:select`, `:multi_select`, `:multi_checkboxes`, `:boolean`, `:date_range`, `:number_range`","title":"Filter Types - Examples","ref":"examples.html#filter-types"},{"type":"extras","doc":"For multiple selection filtering, choose between:\n\n- **`:multi_select`** - Modern tag-based interface with dropdown (default for arrays)\n- **`:multi_checkboxes`** - Traditional checkbox interface","title":"Multi-Select Options - Examples","ref":"examples.html#multi-select-options"},{"type":"extras","doc":"```elixir\n \n  <!-- Basic text filter -->\n  <:col :let={article} field=\"title\" filter>{article.title}</:col>\n\n  <!-- Text filter with custom placeholder -->\n  <:col\n    :let={article}\n    field=\"content\"\n    filter={:text}\n    filter_options={[placeholder: \"Search article content...\"]}\n  >\n    {String.slice(article.content, 0, 100)}...\n  </:col>\n\n  <!-- Case-sensitive text filter -->\n  <:col\n    :let={article}\n    field=\"author_name\"\n    filter={:text}\n    filter_options={[\n      placeholder: \"Author name...\",\n      case_sensitive: true\n    ]}\n  >\n    {article.author_name}\n  </:col>\n \n```","title":"Text Filter - Examples","ref":"examples.html#text-filter"},{"type":"extras","doc":"```elixir\n \n  <!-- Basic select filter (auto-detects enum options) -->\n  <:col :let={order} field=\"status\" filter>{String.capitalize(order.status)}</:col>\n\n  <!-- Select filter with custom options -->\n  <:col\n    :let={order}\n    field=\"priority\"\n    filter={:select}\n    filter_options={[\n      options: [\n        {\"Low Priority\", \"low\"},\n        {\"Normal Priority\", \"normal\"},\n        {\"High Priority\", \"high\"},\n        {\"Urgent\", \"urgent\"}\n      ],\n      prompt: \"Any Priority\"\n    ]}\n  >\n     \n      {String.capitalize(order.priority)}\n     \n  </:col>\n\n  <!-- Select with boolean options -->\n  <:col\n    :let={order}\n    field=\"is_paid\"\n    filter={:select}\n    filter_options={[\n      options: [{\"Paid\", true}, {\"Unpaid\", false}],\n      prompt: \"Payment Status\"\n    ]}\n  >\n    {if order.is_paid, do: \"Paid\", else: \"Unpaid\"}\n  </:col>\n \n```","title":"Select Filter - Examples","ref":"examples.html#select-filter"},{"type":"extras","doc":"```elixir\n \n  <!-- Multi-select for tags -->\n  <:col\n    field=\"tags\"\n    filter={:multi_select}\n    filter_options={[\n      options: [\n        {\"Fiction\", \"fiction\"},\n        {\"Non-Fiction\", \"non_fiction\"},\n        {\"Science Fiction\", \"sci_fi\"},\n        {\"Romance\", \"romance\"},\n        {\"Mystery\", \"mystery\"},\n        {\"Biography\", \"biography\"}\n      ]\n    ]}\n  >\n    {Enum.join(book.tags, \", \")}\n  </:col>\n\n  <!-- Multi-select for categories -->\n  <:col\n    :let={book}\n    field=\"categories\"\n    filter={:multi_select}\n    filter_options={[\n      options: [\n        {\"Bestseller\", \"bestseller\"},\n        {\"New Release\", \"new_release\"},\n        {\"Award Winner\", \"award_winner\"},\n        {\"Staff Pick\", \"staff_pick\"}\n      ]\n    ]}\n  >\n     \n      {for category <- book.categories do}\n         \n          {String.capitalize(String.replace(category, \"_\", \" \"))}\n         \n      {/for}\n     \n  </:col>\n \n```","title":"Multi-Select Filter - Examples","ref":"examples.html#multi-select-filter"},{"type":"extras","doc":"```elixir\n \n  <!-- Basic boolean filter -->\n  <:col :let={user} field=\"is_active\" filter>\n    {if user.is_active, do: \"Active\", else: \"Inactive\"}\n  </:col>\n\n  <!-- Boolean filter with custom labels -->\n  <:col\n    :let={user}\n    field=\"email_verified\"\n    filter={:boolean}\n    filter_options={[\n      labels: %{\n        all: \"Any Verification Status\",\n        true: \"Email Verified\",\n        false: \"Email Not Verified\"\n      }\n    ]}\n  >\n     \n      {if user.email_verified, do: \"Verified\", else: \"Not Verified\"}\n     \n  </:col>\n\n  <!-- Boolean filter for subscription -->\n  <:col\n    :let={user}\n    field=\"has_subscription\"\n    filter={:boolean}\n    filter_options={[\n      labels: %{\n        all: \"All Users\",\n        true: \"Subscribers\",\n        false: \"Free Users\"\n      }\n    ]}\n  >\n    {if user.has_subscription, do: \"Subscriber\", else: \"Free User\"}\n  </:col>\n \n```","title":"Boolean Filter - Examples","ref":"examples.html#boolean-filter"},{"type":"extras","doc":"```elixir\n \n  <!-- Basic date range filter -->\n  <:col :let={event} field=\"created_at\" filter={:date_range}>\n    {Calendar.strftime(event.created_at, \"%B %d, %Y\")}\n  </:col>\n\n  <!-- Date range with custom format -->\n  <:col\n    :let={event}\n    field=\"event_date\"\n    filter={:date_range}\n    filter_options={[\n      format: \"YYYY-MM-DD\",\n      placeholder_from: \"Start date\",\n      placeholder_to: \"End date\"\n    ]}\n  >\n    {Calendar.strftime(event.event_date, \"%Y-%m-%d\")}\n  </:col>\n\n  <!-- DateTime range filter -->\n  <:col\n    :let={event}\n    field=\"updated_at\"\n    filter={:date_range}\n    filter_options={[\n      include_time: true\n    ]}\n  >\n    {Calendar.strftime(event.updated_at, \"%B %d, %Y at %I:%M %p\")}\n  </:col>\n \n```","title":"Date Range Filter - Examples","ref":"examples.html#date-range-filter"},{"type":"extras","doc":"```elixir\n \n  <!-- Basic number range -->\n  <:col :let={property} field=\"price\" filter={:number_range}>\n    ${Number.Currency.number_to_currency(property.price)}\n  </:col>\n\n  <!-- Number range with min/max limits -->\n  <:col\n    :let={property}\n    field=\"square_feet\"\n    filter={:number_range}\n    filter_options={[\n      min: 500,\n      max: 10000,\n      step: 100,\n      placeholder_min: \"Min sq ft\",\n      placeholder_max: \"Max sq ft\"\n    ]}\n  >\n    {Number.Delimit.number_to_delimited(property.square_feet)} sq ft\n  </:col>\n\n  <!-- Decimal number range -->\n  <:col\n    :let={property}\n    field=\"rating\"\n    filter={:number_range}\n    filter_options={[\n      min: 0.0,\n      max: 5.0,\n      step: 0.1\n    ]}\n  >\n     \n      {for i <- 1..5 do}\n          property.rating && \"text-gray-300\"\n        ]} fill=\"currentColor\" viewBox=\"0 0 20 20\">\n            \n         \n      {/for}\n       {property.rating} \n     \n  </:col>\n \n```","title":"Number Range Filter - Examples","ref":"examples.html#number-range-filter"},{"type":"extras","doc":"","title":"Sorting - Examples","ref":"examples.html#sorting"},{"type":"extras","doc":"```elixir\n \n  <!-- Sortable columns -->\n  <:col :let={user} field=\"name\" sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" sort>{user.email}</:col>\n  <:col :let={user} field=\"created_at\" sort>{user.created_at}</:col>\n\n  <!-- Non-sortable column -->\n  <:col :let={user} field=\"bio\">{user.bio}</:col>\n \n```","title":"Basic Sorting - Examples","ref":"examples.html#basic-sorting"},{"type":"extras","doc":"```elixir\n \n  <:col :let={product} field=\"name\" filter sort>{product.name}</:col>\n  <:col :let={product} field=\"price\" filter={:number_range} sort>${product.price}</:col>\n  <:col :let={product} field=\"category\" filter={:select} sort>{product.category}</:col>\n  <:col :let={product} field=\"created_at\" filter={:date_range} sort>{product.created_at}</:col>\n \n```","title":"Combined Filter and Sort - Examples","ref":"examples.html#combined-filter-and-sort"},{"type":"extras","doc":"","title":"Theming - Examples","ref":"examples.html#theming"},{"type":"extras","doc":"```elixir\n<!-- Default theme -->\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n \n\n<!-- Modern theme -->\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n \n\n<!-- Minimal theme -->\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n \n```","title":"Built-in Themes - Examples","ref":"examples.html#built-in-themes"},{"type":"extras","doc":"```elixir\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" filter sort>{user.email}</:col>\n \n```","title":"Custom Theme - Complete Example - Examples","ref":"examples.html#custom-theme-complete-example"},{"type":"extras","doc":"","title":"URL State Management - Examples","ref":"examples.html#url-state-management"},{"type":"extras","doc":"```elixir\ndefmodule MyAppWeb.UsersLive do\n  use MyAppWeb, :live_view\n  use Cinder.Table.UrlSync\n\n  def mount(_params, _session, socket) do\n    current_user = get_current_user(socket)\n    {:ok, assign(socket, :current_user, current_user)}\n  end\n\n  def handle_params(params, uri, socket) do\n    socket = Cinder.Table.UrlSync.handle_params(params, uri, socket)\n    {:noreply, socket}\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n       Users \n\n       \n        <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n        <:col :let={user} field=\"email\" filter>{user.email}</:col>\n        <:col :let={user} field=\"department.name\" filter sort>{user.department.name}</:col>\n        <:col :let={user} field=\"is_active\" filter={:boolean}>\n          {if user.is_active, do: \"Active\", else: \"Inactive\"}\n        </:col>\n       \n     \n    \"\"\"\n  end\nend\n```","title":"Complete LiveView Setup - Examples","ref":"examples.html#complete-liveview-setup"},{"type":"extras","doc":"You can also use pre-configured queries with URL sync:\n\n```elixir\ndefmodule MyAppWeb.ActiveUsersLive do\n  use MyAppWeb, :live_view\n  use Cinder.Table.UrlSync\n\n  def mount(_params, _session, socket) do\n    current_user = get_current_user(socket)\n    {:ok, assign(socket, :current_user, current_user)}\n  end\n\n  def handle_params(params, uri, socket) do\n    socket = Cinder.Table.UrlSync.handle_params(params, uri, socket)\n    {:noreply, socket}\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n       Active Users \n\n        Ash.Query.filter(active: true)}\n        actor={@current_user}\n        url_state={@url_state}\n        id=\"active-users-table\"\n      >\n        <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n        <:col :let={user} field=\"email\" filter>{user.email}</:col>\n        <:col :let={user} field=\"last_login\" sort>{user.last_login}</:col>\n       \n     \n    \"\"\"\n  end\nend\n```","title":"URL State Management with Query Parameter - Examples","ref":"examples.html#url-state-management-with-query-parameter"},{"type":"extras","doc":"With URL sync enabled, your table state is preserved in the URL:\n\n```\n# Basic filtering\n/users?name=john&department.name=engineering\n\n# With date range\n/users?name=smith&created_at_from=2024-01-01&created_at_to=2024-12-31\n\n# With pagination and sorting\n/users?email=gmail&page=3&sort=-created_at\n\n# Complex state\n/users?name=admin&department.name=IT&is_active=true&page=2&sort=name,-created_at\n```","title":"URL Examples - Examples","ref":"examples.html#url-examples"},{"type":"extras","doc":"","title":"Relationship Fields - Examples","ref":"examples.html#relationship-fields"},{"type":"extras","doc":"```elixir\n \n  <:col :let={album} field=\"title\" filter sort>{album.title}</:col>\n  <:col :let={album} field=\"artist.name\" filter sort>{album.artist.name}</:col>\n  <:col :let={album} field=\"artist.country\" filter>{album.artist.country}</:col>\n  <:col :let={album} field=\"record_label.name\" filter>{album.record_label.name}</:col>\n \n```","title":"Basic Relationships - Examples","ref":"examples.html#basic-relationships"},{"type":"extras","doc":"```elixir\n \n  <:col :let={employee} field=\"name\" filter sort>{employee.name}</:col>\n  <:col :let={employee} field=\"department.name\" filter sort>{employee.department.name}</:col>\n  <:col :let={employee} field=\"department.manager.name\" filter>{employee.department.manager.name}</:col>\n  <:col :let={employee} field=\"office.building.address\" filter>{employee.office.building.address}</:col>\n \n```","title":"Deep Relationships - Examples","ref":"examples.html#deep-relationships"},{"type":"extras","doc":"```elixir\n \n  <:col :let={order} field=\"number\" filter sort>#{order.number}</:col>\n  <:col :let={order} field=\"customer.name\" filter sort>{order.customer.name}</:col>\n\n  <!-- Select filter on relationship enum -->\n  <:col\n    :let={order}\n    field=\"customer.tier\"\n    filter={:select}\n    filter_options={[\n      options: [{\"Bronze\", \"bronze\"}, {\"Silver\", \"silver\"}, {\"Gold\", \"gold\"}],\n      prompt: \"Any Tier\"\n    ]}\n  >\n     \n      {String.capitalize(order.customer.tier)}\n     \n  </:col>\n\n  <!-- Date range on relationship -->\n  <:col\n    :let={order}\n    field=\"customer.created_at\"\n    filter={:date_range}\n    sort\n  >\n    {Calendar.strftime(order.customer.created_at, \"%B %Y\")}\n  </:col>\n \n```","title":"Relationship Filters with Custom Options - Examples","ref":"examples.html#relationship-filters-with-custom-options"},{"type":"extras","doc":"#### Progress Bars and Indicators\n\n```elixir\n \n  <:col :let={project} field=\"name\" filter sort>\n    {project.name}\n  </:col>\n\n  <!-- Progress bar -->\n  <:col :let={project} field=\"completion_percentage\" filter={:number_range} sort>\n     \n       \n         \n         \n       \n       \n        {project.completion_percentage}%\n       \n     \n  </:col>\n\n  <!-- Health indicator -->\n  <:col :let={project} field=\"health_status\" filter={:select}>\n     \n       \n       \n       \n        {project.health_status}\n       \n     \n  </:col>\n \n```\n\n#### Image Thumbnails and Rich Content\n\n```elixir\n \n  <!-- Product with thumbnail -->\n  <:col :let={product} field=\"name\" filter sort class=\"w-1/3\">\n     \n      {if product.image_url do}\n         \n      {else}\n         \n           \n              \n           \n         \n      {/if}\n       \n         \n          {product.name}\n         \n         \n          SKU: {product.sku}\n         \n       \n     \n  </:col>\n\n  <!-- Price with currency formatting -->\n  <:col :let={product} field=\"price\" filter={:number_range} sort class=\"text-right\">\n     \n       \n        {Money.to_string(product.price)}\n       \n      {if product.sale_price do}\n         \n          {Money.to_string(product.sale_price)}\n         \n      {/if}\n     \n  </:col>\n\n  <!-- Stock status with inventory count -->\n  <:col :let={product} field=\"inventory_count\" filter={:number_range} sort>\n     \n        50 && \"bg-green-100 text-green-800\",\n        product.inventory_count > 10 && product.inventory_count <= 50 && \"bg-yellow-100 text-yellow-800\",\n        product.inventory_count <= 10 && \"bg-red-100 text-red-800\"\n      ]}>\n        {product.inventory_count} in stock\n       \n     \n  </:col>\n \n```","title":"Advanced Examples - Examples","ref":"examples.html#advanced-examples"},{"type":"extras","doc":"","title":"Advanced Configuration - Examples","ref":"examples.html#advanced-configuration"},{"type":"extras","doc":"Every available option demonstrated:\n\n```elixir\n \n  <!-- Text column with all options using :let -->\n  <:col\n    :let={user}\n    field=\"name\"\n    label=\"Full Name\"\n    filter={:text}\n    filter_options={[\n      placeholder: \"Search by name...\",\n      case_sensitive: false\n    ]}\n    sort={true}\n    class=\"w-1/4 font-semibold\"\n  >\n     \n       \n         \n          {String.first(user.name)}\n         \n       \n       {user.name} \n     \n  </:col>\n\n  <!-- Email with mailto link using :let -->\n  <:col\n    :let={user}\n    field=\"email\"\n    filter={:text}\n    filter_options={[placeholder: \"Email address...\"]}\n    sort\n    class=\"w-1/4\"\n  >\n     \n      {user.email}\n     \n  </:col>\n\n  <!-- Relationship with select filter -->\n  <:col\n    :let={user}\n    field=\"department.name\"\n    filter={:select}\n    filter_options={[\n      options: [\n        {\"Engineering\", \"engineering\"},\n        {\"Marketing\", \"marketing\"},\n        {\"Sales\", \"sales\"},\n        {\"Support\", \"support\"}\n      ],\n      prompt: \"All Departments\"\n    ]}\n    sort\n  >\n    {user.department.name}\n  </:col>\n\n  <!-- Date with range filter using :let -->\n  <:col\n    :let={user}\n    field=\"created_at\"\n    label=\"Member Since\"\n    filter={:date_range}\n    filter_options={[\n      format: \"MM/DD/YYYY\",\n      placeholder_from: \"Start date\",\n      placeholder_to: \"End date\"\n    ]}\n    sort\n  >\n     \n       \n        {Calendar.strftime(user.created_at, \"%B %d, %Y\")}\n       \n       \n        {Calendar.strftime(user.created_at, \"%H:%M\")}\n       \n     \n  </:col>\n\n  <!-- Boolean with custom labels using :let -->\n  <:col\n    :let={user}\n    field=\"is_active\"\n    filter={:boolean}\n    filter_options={[\n      labels: %{\n        all: \"All Users\",\n        true: \"Active Users\",\n        false: \"Inactive Users\"\n      }\n    ]}\n  >\n     \n      {if user.is_active, do: \"Active\", else: \"Inactive\"}\n     \n  </:col>\n\n  <!-- Actions column using :let -->\n  <:col :let={user} field=\"actions\" class=\"text-right w-32\">\n     \n      <.link\n        navigate={~p\"/users/#{user.id}\"}\n        class=\"p-1 text-blue-600 hover:text-blue-800\"\n        title=\"View user\"\n      >\n         \n            \n            \n         \n      </.link>\n      <.link\n        navigate={~p\"/users/#{user.id}/edit\"}\n        class=\"p-1 text-green-600 hover:text-green-800\"\n        title=\"Edit user\"\n      >\n         \n            \n         \n      </.link>\n     \n  </:col>\n \n```","title":"Complete Configuration Example - Examples","ref":"examples.html#complete-configuration-example"},{"type":"extras","doc":"","title":"Performance Optimization - Examples","ref":"examples.html#performance-optimization"},{"type":"extras","doc":"```elixir\n \n  <:col field=\"number\" filter sort>Order #</:col>\n  <:col field=\"customer.name\" filter sort>Customer</:col>\n  <:col field=\"total_amount\" filter={:number_range} sort>Total</:col>\n \n```","title":"Efficient Data Loading - Examples","ref":"examples.html#efficient-data-loading"},{"type":"extras","doc":"Only enable filters where users actually need them:\n\n```elixir\n \n  <!-- Internal ID - no filter needed -->\n  <:col :let={product} field=\"id\" sort>{product.id}</:col>\n\n  <!-- User-searchable fields -->\n  <:col :let={product} field=\"name\" filter sort>{product.name}</:col>\n  <:col :let={product} field=\"category\" filter sort>{product.category}</:col>\n  <:col :let={product} field=\"price\" filter={:number_range} sort>${product.price}</:col>\n\n  <!-- Display-only field -->\n  <:col :let={product} field=\"sku\">{product.sku}</:col>\n \n```","title":"Strategic Filtering - Examples","ref":"examples.html#strategic-filtering"},{"type":"extras","doc":"```elixir\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" filter>{user.email}</:col>\n  <:col :let={user} field=\"department.name\" filter sort>{user.department.name}</:col>\n \n```\n\nThis comprehensive guide demonstrates every available feature and option in Cinder. The combination of intelligent defaults and extensive customization options makes Cinder suitable for simple data display as well as complex, feature-rich table implementations.","title":"Custom Query Optimization - Examples","ref":"examples.html#custom-query-optimization"},{"type":"extras","doc":"# Theming\n\nCinder provides a comprehensive theming system that allows complete visual customization of your tables. With 10 built-in themes and a powerful DSL for creating custom themes, you can match any design system or create unique visual experiences.\n\n> **See Also**: [Theme Showcase](theme-showcase.md) - Visual examples and comparisons of all built-in themes","title":"Theming","ref":"theming.html"},{"type":"extras","doc":"- [Quick Start](#quick-start)\n- [Built-in Theme Presets](#built-in-theme-presets)\n- [Custom Themes with DSL](#custom-themes-with-dsl)\n- [Theme Inheritance](#theme-inheritance)\n- [Developer Tools](#developer-tools)\n- [Component Reference](#component-reference)","title":"Table of Contents - Theming","ref":"theming.html#table-of-contents"},{"type":"extras","doc":"","title":"Quick Start - Theming","ref":"theming.html#quick-start"},{"type":"extras","doc":"The fastest way to style your table is with one of the 10 built-in themes:\n\n```elixir\n \n  <:col field=\"name\" filter sort>Name</:col>\n  <:col field=\"email\" filter>Email</:col>\n \n```","title":"Using Built-in Themes - Theming","ref":"theming.html#using-built-in-themes"},{"type":"extras","doc":"Create reusable themes with the Cinder DSL:\n\n```elixir\ndefmodule MyApp.CustomTheme do\n  use Cinder.Theme\n\n  component Cinder.Components.Table do\n    set :container_class, \"bg-white shadow-lg rounded-lg border\"\n    set :th_class, \"px-6 py-4 bg-gray-50 font-semibold text-gray-900\"\n    set :row_class, \"hover:bg-gray-50 transition-colors\"\n  end\nend\n\n# Use in your template\n \n  <:col field=\"name\" filter sort>Name</:col>\n \n```","title":"Custom Theme Module - Theming","ref":"theming.html#custom-theme-module"},{"type":"extras","doc":"Cinder includes 10 carefully crafted themes covering a wide range of design styles. Each theme provides complete coverage for all table components while maintaining a consistent visual identity.\n\n> **Visual Reference**: See the [Theme Showcase](theme-showcase.md) for detailed visual examples and feature descriptions of each theme.\n\nAvailable themes:\n\n- **`\"default\"`** - Clean, minimal styling for universal compatibility\n- **`\"modern\"`** - Professional styling with shadows and improved spacing\n- **`\"dark\"`** - Elegant dark theme with proper contrast\n- **`\"daisy_ui\"`** - Optimized for DaisyUI component library\n- **`\"flowbite\"`** - Designed for Flowbite design system\n- **`\"retro\"`** - Cyberpunk-inspired with bright accent colors\n- **`\"futuristic\"`** - Sci-fi aesthetic with glowing effects\n- **`\"compact\"`** - High-density layout for data-heavy applications\n- **`\"pastel\"`** - Soft, friendly colors for approachable interfaces","title":"Built-in Theme Presets - Theming","ref":"theming.html#built-in-theme-presets"},{"type":"extras","doc":"```elixir\n<!-- Use any theme by name -->\n \n  <:col :let={user} field=\"name\" filter sort>{user.name}</:col>\n  <:col :let={user} field=\"email\" filter>{user.email}</:col>\n \n```","title":"Usage - Theming","ref":"theming.html#usage"},{"type":"extras","doc":"Create powerful, maintainable themes using Cinder's DSL syntax:","title":"Custom Themes with DSL - Theming","ref":"theming.html#custom-themes-with-dsl"},{"type":"extras","doc":"```elixir\ndefmodule MyApp.Theme.Corporate do\n  use Cinder.Theme\n\n  component Cinder.Components.Table do\n    set :container_class, \"bg-white shadow-lg rounded-lg border border-gray-200\"\n    set :th_class, \"px-6 py-4 bg-blue-50 text-left font-semibold text-blue-900\"\n    set :td_class, \"px-6 py-4 border-b border-gray-100 text-gray-900\"\n    set :row_class, \"hover:bg-blue-50 transition-colors duration-150\"\n  end\n\n  component Cinder.Components.Filters do\n    set :filter_container_class, \"bg-blue-50 border border-blue-200 rounded-lg p-6 mb-6\"\n    set :filter_title_class, \"text-lg font-semibold text-blue-900 mb-4\"\n    set :filter_text_input_class, \"w-full px-4 py-3 border border-blue-300 rounded-lg focus:ring-2 focus:ring-blue-500\"\n  end\n\n  component Cinder.Components.Pagination do\n    set :pagination_button_class, \"px-4 py-2 bg-blue-600 text-white rounded-lg hover:bg-blue-700 transition-colors\"\n    set :pagination_info_class, \"text-blue-700 font-medium\"\n  end\nend\n```","title":"Basic Theme Structure - Theming","ref":"theming.html#basic-theme-structure"},{"type":"extras","doc":"Customize only the components you need:\n\n```elixir\ndefmodule MyApp.Theme.FilterFocused do\n  use Cinder.Theme\n\n  # Only customize filters, leave table and pagination with defaults\n  component Cinder.Components.Filters do\n    set :filter_container_class, \"bg-gradient-to-r from-purple-50 to-pink-50 border-2 border-purple-200 rounded-xl p-8 mb-8\"\n    set :filter_title_class, \"text-xl font-bold text-purple-900 mb-6\"\n    set :filter_text_input_class, \"w-full px-4 py-3 border-2 border-purple-300 rounded-lg focus:ring-4 focus:ring-purple-200\"\n    set :filter_boolean_container_class, \"flex space-x-6 bg-white p-4 rounded-lg shadow-sm\"\n    set :filter_boolean_radio_class, \"h-5 w-5 text-purple-600 focus:ring-purple-500\"\n  end\nend\n```","title":"Component-Specific Customization - Theming","ref":"theming.html#component-specific-customization"},{"type":"extras","doc":"Build upon existing themes using the `extends` directive:","title":"Theme Inheritance - Theming","ref":"theming.html#theme-inheritance"},{"type":"extras","doc":"```elixir\ndefmodule MyApp.Theme.DarkModern do\n  use Cinder.Theme\n  extends :modern\n\n  component Cinder.Components.Table do\n    set :container_class, \"bg-gray-900 shadow-xl rounded-lg border border-gray-700\"\n    set :th_class, \"px-6 py-4 bg-gray-800 text-left font-semibold text-gray-100 border-b border-gray-700\"\n    set :td_class, \"px-6 py-4 text-gray-200 border-b border-gray-700\"\n    set :row_class, \"hover:bg-gray-800 transition-colors\"\n  end\n\n  component Cinder.Components.Filters do\n    set :filter_container_class, \"bg-gray-800 border border-gray-700 rounded-lg p-6 mb-6\"\n    set :filter_title_class, \"text-sm font-medium text-gray-200\"\n    set :filter_text_input_class, \"w-full px-3 py-2 bg-gray-700 border border-gray-600 rounded-md text-gray-200 focus:ring-2 focus:ring-blue-500\"\n  end\nend\n```","title":"Extending Built-in Themes - Theming","ref":"theming.html#extending-built-in-themes"},{"type":"extras","doc":"```elixir\ndefmodule MyApp.Theme.CorporateCompact do\n  use Cinder.Theme\n  extends MyApp.Theme.Corporate\n\n  # Make the corporate theme more compact\n  component Cinder.Components.Table do\n    set :th_class, \"px-4 py-2 bg-blue-50 text-left font-semibold text-blue-900 border-b border-blue-200\"\n    set :td_class, \"px-4 py-2 border-b border-gray-100 text-gray-900\"\n  end\n\n  component Cinder.Components.Filters do\n    set :filter_container_class, \"bg-blue-50 border border-blue-200 rounded-lg p-4 mb-4\"\n  end\nend\n```","title":"Extending Custom Themes - Theming","ref":"theming.html#extending-custom-themes"},{"type":"extras","doc":"Cinder includes built-in developer tools to make theme creation effortless:","title":"Developer Tools - Theming","ref":"theming.html#developer-tools"},{"type":"extras","doc":"Every themed element includes a `data-key` attribute identifying which theme property controls it:\n\n```html\n \n   \n     \n       \n         Name \n       \n     \n   \n \n```","title":"Data Attributes - Theming","ref":"theming.html#data-attributes"},{"type":"extras","doc":"1. **Inspect any element** in your table\n2. **Look for the `data-key` attribute** to see which theme property controls it\n3. **Update your theme** with the identified property name\n4. **See changes immediately** without guessing\n\nExample workflow:\n```bash\n# 1. Inspect element in browser\n \n\n# 2. Update your theme\ncomponent Cinder.Components.Filters do\n  set :filter_text_input_class, \"w-full px-4 py-3 border-2 border-blue-500 rounded-lg\"\nend\n\n# 3. Refresh to see changes\n```","title":"Using Browser Dev Tools - Theming","ref":"theming.html#using-browser-dev-tools"},{"type":"extras","doc":"Cinder organizes theme properties by logical components:","title":"Component Reference - Theming","ref":"theming.html#component-reference"},{"type":"extras","doc":"Controls the main table structure:\n\n```elixir\ncomponent Cinder.Components.Table do\n  set :container_class, \"...\"           # Outer container\n  set :controls_class, \"...\"            # Controls section\n  set :table_wrapper_class, \"...\"       # Scrollable wrapper\n  set :table_class, \"...\"               # Table element\n  set :thead_class, \"...\"               # Table header\n  set :tbody_class, \"...\"               # Table body\n  set :header_row_class, \"...\"          # Header row\n  set :row_class, \"...\"                 # Data rows\n  set :th_class, \"...\"                  # Header cells\n  set :td_class, \"...\"                  # Data cells\n  set :loading_class, \"...\"             # Loading state\n  set :empty_class, \"...\"               # Empty state\n  set :error_container_class, \"...\"     # Error container\n  set :error_message_class, \"...\"       # Error message\nend\n```","title":"Table Component - Theming","ref":"theming.html#table-component"},{"type":"extras","doc":"Controls all filter-related styling:\n\n```elixir\ncomponent Cinder.Components.Filters do\n  # Container structure\n  set :filter_container_class, \"...\"\n  set :filter_header_class, \"...\"\n  set :filter_title_class, \"...\"\n  set :filter_count_class, \"...\"\n  set :filter_clear_all_class, \"...\"\n  set :filter_inputs_class, \"...\"\n  set :filter_input_wrapper_class, \"...\"\n  set :filter_label_class, \"...\"\n  set :filter_placeholder_class, \"...\"\n  set :filter_clear_button_class, \"...\"\n\n  # Input types\n  set :filter_text_input_class, \"...\"\n  set :filter_select_input_class, \"...\"\n  set :filter_date_input_class, \"...\"\n  set :filter_number_input_class, \"...\"\n\n  # Boolean filters\n  set :filter_boolean_container_class, \"...\"\n  set :filter_boolean_option_class, \"...\"\n  set :filter_boolean_radio_class, \"...\"\n  set :filter_boolean_label_class, \"...\"\n\n  # Multi-select filters (dropdown interface)\n  set :filter_multiselect_container_class, \"...\"\n  set :filter_multiselect_dropdown_class, \"...\"\n  set :filter_multiselect_option_class, \"...\"\n  set :filter_multiselect_checkbox_class, \"...\"\n  set :filter_multiselect_label_class, \"...\"\n  set :filter_multiselect_empty_class, \"...\"\n\n  # Multi-checkboxes filters (traditional interface)\n  set :filter_multicheckboxes_container_class, \"...\"\n  set :filter_multicheckboxes_option_class, \"...\"\n  set :filter_multicheckboxes_checkbox_class, \"...\"\n  set :filter_multicheckboxes_label_class, \"...\"\n\n  # Range filters\n  set :filter_range_container_class, \"...\"\n  set :filter_range_input_group_class, \"...\"\n  set :filter_range_separator_class, \"...\"\nend\n```","title":"Filters Component - Theming","ref":"theming.html#filters-component"},{"type":"extras","doc":"Controls pagination styling:\n\n```elixir\ncomponent Cinder.Components.Pagination do\n  set :pagination_wrapper_class, \"...\"    # Outer wrapper\n  set :pagination_container_class, \"...\"  # Inner container\n  set :pagination_info_class, \"...\"       # Page info (\"Page 1 of 10\")\n  set :pagination_nav_class, \"...\"        # Navigation section\n  set :pagination_button_class, \"...\"     # Navigation buttons\n  set :pagination_current_class, \"...\"    # Current page indicator\n  set :pagination_count_class, \"...\"      # Record count\nend\n```","title":"Pagination Component - Theming","ref":"theming.html#pagination-component"},{"type":"extras","doc":"Controls sort indicators and icons:\n\n```elixir\ncomponent Cinder.Components.Sorting do\n  set :sort_indicator_class, \"...\"         # Sort indicator wrapper\n  set :sort_arrow_wrapper_class, \"...\"     # Icon wrapper\n  set :sort_asc_icon_class, \"...\"          # Ascending icon\n  set :sort_asc_icon_name, \"hero-chevron-up\"     # Icon name\n  set :sort_desc_icon_class, \"...\"         # Descending icon\n  set :sort_desc_icon_name, \"hero-chevron-down\"  # Icon name\n  set :sort_none_icon_class, \"...\"         # Unsorted icon\n  set :sort_none_icon_name, \"hero-chevron-up-down\"  # Icon name\nend\n```","title":"Sorting Component - Theming","ref":"theming.html#sorting-component"},{"type":"extras","doc":"Controls loading states:\n\n```elixir\ncomponent Cinder.Components.Loading do\n  set :loading_overlay_class, \"...\"        # Loading overlay\n  set :loading_container_class, \"...\"      # Loading container\n  set :loading_class, \"...\"                # Loading text\n  set :loading_spinner_class, \"...\"        # Spinner element\n  set :loading_spinner_circle_class, \"...\" # Spinner circle\n  set :loading_spinner_path_class, \"...\"   # Spinner path\nend\n```","title":"Loading Component - Theming","ref":"theming.html#loading-component"},{"type":"extras","doc":"# Theme Showcase\n\nThis document showcases the dramatic visual variety possible with Cinder's theming system. Each theme completely transforms the appearance while maintaining full functionality.\n\n> **Getting Started**: For technical documentation on creating and customizing themes, see the [Theming Guide](theming.md).","title":"Theme Showcase","ref":"theme-showcase.html"},{"type":"extras","doc":"The default theme provides only essential usability classes - no styling, perfect for integration with any CSS framework.\n\n```elixir\n# No theme specified = minimal default\n \n  <:col field=\"name\" />\n  <:col field=\"email\" />\n  <:col field=\"role\" />\n \n```\n\n**Classes applied**: Only `overflow-x-auto`, `w-full border-collapse`, `text-left whitespace-nowrap`","title":"Default Theme (Minimal) - Theme Showcase","ref":"theme-showcase.html#default-theme-minimal"},{"type":"extras","doc":"Professional, clean design with subtle shadows and blue accents.\n\n```elixir\n \n  <:col field=\"name\" />\n  <:col field=\"email\" />\n  <:col field=\"role\" />\n \n```\n\n**Visual Features**:\n- White background with rounded corners and subtle shadows\n- Blue gradient headers with professional typography\n- Smooth hover transitions\n- Modern form inputs with focus states\n- Clean pagination with border styling\n\n![Screenshot of modern theme](./screenshots/modern.png)","title":"Modern Theme - Theme Showcase","ref":"theme-showcase.html#modern-theme"},{"type":"extras","doc":"Bold 80s-inspired design with neon colors and cyberpunk aesthetics.\n\n```elixir\n \n  <:col field=\"name\" />\n  <:col field=\"email\" />\n  <:col field=\"role\" />\n \n```\n\n**Visual Features**:\n- Dark backgrounds with bright cyan/magenta borders\n- Neon color accents (cyan, magenta, yellow)\n- Bold uppercase typography with wide letter spacing\n- Glowing effects on interactive elements\n- High contrast design for dramatic impact\n\n![Screenshot of retro theme](./screenshots/retro.png)","title":"Retro Theme - Theme Showcase","ref":"theme-showcase.html#retro-theme"},{"type":"extras","doc":"Sci-fi inspired with holographic effects and translucent elements.\n\n```elixir\n \n  <:col field=\"name\" />\n  <:col field=\"email\" />\n  <:col field=\"role\" />\n \n```\n\n**Visual Features**:\n- Dark space-like backgrounds with subtle gradients\n- Blue and green holographic color scheme\n- Backdrop blur effects for depth\n- Translucent overlays and borders\n- Light typography with wide tracking\n\n![Screenshot of futuristic theme](./screenshots/futuristic.png)","title":"Futuristic Theme - Theme Showcase","ref":"theme-showcase.html#futuristic-theme"},{"type":"extras","doc":"Elegant dark mode with purple accents and smooth gradients.\n\n```elixir\n \n  <:col field=\"name\" />\n  <:col field=\"email\" />\n  <:col field=\"role\" />\n \n```\n\n**Visual Features**:\n- Rich dark gray backgrounds\n- Purple and blue accent colors\n- High contrast for excellent readability\n- Smooth color transitions\n- Professional dark mode aesthetic\n\n![Screenshot of dark theme](./screenshots/dark.png)","title":"Dark Theme - Theme Showcase","ref":"theme-showcase.html#dark-theme"},{"type":"extras","doc":"Complete compatibility with daisyUI component library.\n\n```elixir\n \n  <:col field=\"name\" />\n  <:col field=\"email\" />\n  <:col field=\"role\" />\n \n```\n\n**Visual Features**:\n- Uses semantic daisyUI classes (`card`, `btn`, `table-zebra`)\n- Compatible with daisyUI color themes\n- Form inputs use daisyUI input/select classes\n- Pagination with daisyUI button styling\n- Automatic theme switching support\n\n![Screenshot of DaisyUI theme](./screenshots/daisy_ui.png)","title":"DaisyUI Theme - Theme Showcase","ref":"theme-showcase.html#daisyui-theme"},{"type":"extras","doc":"Matches Flowbite's advanced table design system.\n\n```elixir\n \n  <:col field=\"name\" />\n  <:col field=\"email\" />\n  <:col field=\"role\" />\n \n```\n\n**Visual Features**:\n- Gray color scheme with Flowbite's exact styling\n- Dark mode support with proper dark: classes\n- Professional table borders and spacing\n- Flowbite form input styling\n- Advanced pagination matching Flowbite docs\n\n![Screenshot of Flowbite theme](./screenshots/flowbite.png)","title":"Flowbite Theme - Theme Showcase","ref":"theme-showcase.html#flowbite-theme"},{"type":"extras","doc":"High-density design with minimal spacing for maximum information display.\n\n```elixir\n \n  <:col field=\"name\" />\n  <:col field=\"email\" />\n  <:col field=\"role\" />\n \n```\n\n**Visual Features**:\n- Minimal padding and margins\n- Smaller text sizes for information density\n- Efficient use of space\n- Clean lines and subtle borders\n- Perfect for dashboards and data-heavy interfaces\n\n![Screenshot of compact theme](./screenshots/compact.png)","title":"Compact Theme - Theme Showcase","ref":"theme-showcase.html#compact-theme"},{"type":"extras","doc":"Soft, gentle design with pastel colors for a calming interface.\n\n```elixir\n \n  <:col field=\"name\" />\n  <:col field=\"email\" />\n  <:col field=\"role\" />\n \n```\n\n**Visual Features**:\n- Soft pastel color palette (pinks, blues, purples, greens)\n- Gentle gradients and rounded corners\n- Light, airy feeling with whitespace\n- Smooth transitions and subtle shadows\n- Soothing design for comfortable viewing\n\n![Screenshot of pastel theme](./screenshots/pastel.png)","title":"Pastel Theme - Theme Showcase","ref":"theme-showcase.html#pastel-theme"},{"type":"extras","doc":"Create your own themes using the powerful DSL:\n\n```elixir\ndefmodule MyApp.CustomTheme do\n  use Cinder.Theme\n  extends :modern\n\n  component Cinder.Components.Table do\n    set :container_class, \"bg-gradient-to-r from-purple-900 to-blue-900 rounded-2xl shadow-2xl\"\n    set :th_class, \"px-6 py-4 text-yellow-300 font-bold uppercase tracking-widest\"\n    set :row_class, \"border-b border-purple-500/30 hover:bg-purple-800/50\"\n  end\n\n  component Cinder.Components.Filters do\n    set :filter_container_class, \"bg-black border-2 border-yellow-400 p-6 rounded-lg\"\n    set :filter_text_input_class, \"bg-gray-900 border-yellow-400 text-yellow-100\"\n  end\nend\n\n# Use in template\n \n  <:col field=\"name\" />\n \n```","title":"Custom Themes with DSL - Theme Showcase","ref":"theme-showcase.html#custom-themes-with-dsl"},{"type":"extras","doc":"Extend existing themes and override specific properties:\n\n```elixir\ndefmodule MyApp.DarkRetro do\n  use Cinder.Theme\n  extends :retro\n\n  component Cinder.Components.Table do\n    # Keep retro styling but make it even darker\n    set :container_class, \"bg-black border-4 border-green-400 shadow-green-400/50\"\n    set :th_class, \"px-6 py-4 text-green-100 font-bold bg-green-900 border-b-4 border-green-400\"\n  end\nend\n```","title":"Theme Inheritance - Theme Showcase","ref":"theme-showcase.html#theme-inheritance"},{"type":"extras","doc":"All built-in themes can be used with string names:\n\n```elixir\n# String preset names\ntheme=\"default\"     # Minimal unstyled\ntheme=\"modern\"      # Professional blue theme\ntheme=\"retro\"       # 80s cyberpunk neon\ntheme=\"futuristic\"  # Sci-fi holographic\ntheme=\"dark\"        # Elegant dark mode\ntheme=\"daisy_ui\"    # daisyUI compatibility\ntheme=\"flowbite\"    # Flowbite compatibility\ntheme=\"compact\"     # High-density minimal\ntheme=\"pastel\"      # Soft gentle colors\n\n# Or use theme modules directly\ntheme={Cinder.Themes.Modern}\ntheme={MyApp.CustomTheme}\n```","title":"Available Theme Presets - Theme Showcase","ref":"theme-showcase.html#available-theme-presets"},{"type":"extras","doc":"Every theme covers all components consistently:\n\n- **Table**: Container, headers, rows, cells\n- **Filters**: Container, inputs, labels, buttons\n- **Pagination**: Wrapper, buttons, info text\n- **Sorting**: Icons, indicators, hover states\n- **Loading**: Spinners, overlays, containers\n\nThis ensures a cohesive experience across all table functionality regardless of which theme you choose.","title":"Component Coverage - Theme Showcase","ref":"theme-showcase.html#component-coverage"},{"type":"extras","doc":"# Custom Filters\n\nThis guide covers how to create custom filters for Cinder tables. Custom filters allow you to extend Cinder's filtering capabilities with domain-specific UI components and logic.","title":"Custom Filters","ref":"custom-filters.html"},{"type":"extras","doc":"Cinder's filter system is built around the `Cinder.Filter` behaviour, which defines a standard interface for all filter types. Custom filters implement this behaviour and can be registered with the system for use in tables.","title":"Overview - Custom Filters","ref":"custom-filters.html#overview"},{"type":"extras","doc":"The easiest way to create custom filters is using the `cinder.gen.filter` Mix task. This generator creates a custom filter based on one of the built-in filters, allowing you to customize specific behaviors while keeping the rest of the implementation.","title":"Generator - mix cinder.gen.filter - Custom Filters","ref":"custom-filters.html#generator-mix-cinder-gen-filter"},{"type":"extras","doc":"```bash\nmix cinder.gen.filter MyApp.Filters.CustomText custom_text --template=text\n```\n\nThis creates:\n- A custom filter module that delegates to `Cinder.Filters.Text`\n- Automatically updates your configuration\n- Generates comprehensive test files\n- Updates the filter type to your custom type","title":"Basic Usage - Custom Filters","ref":"custom-filters.html#basic-usage"},{"type":"extras","doc":"- `text` - Based on `Cinder.Filters.Text` (text input with operators)\n- `select` - Based on `Cinder.Filters.Select` (dropdown selection)\n- `multi_select` - Based on `Cinder.Filters.MultiSelect` (multiple selection)\n- `multi_checkboxes` - Based on `Cinder.Filters.MultiCheckboxes` (multiple selection)\n- `boolean` - Based on `Cinder.Filters.Boolean` (true/false/any selection)\n- `date_range` - Based on `Cinder.Filters.DateRange` (from/to date picker)\n- `number_range` - Based on `Cinder.Filters.NumberRange` (from/to number input)","title":"Available Templates - Custom Filters","ref":"custom-filters.html#available-templates"},{"type":"extras","doc":"```bash\nmix cinder.gen.filter MyApp.Filters.CaseInsensitiveText case_insensitive_text --template=text\n```\n\nThis generates a filter that starts by delegating all behavior to `Cinder.Filters.Text` but with your custom type. You can then override specific functions to customize behavior:\n\n```elixir\ndefmodule MyApp.Filters.CaseInsensitiveText do\n  # ... generated delegations ...\n\n  # Override to customize processing\n  @impl true\n  def process(raw_value, column) do\n    case Cinder.Filters.Text.process(raw_value, column) do\n      %{type: _old_type, value: value} = filter ->\n        %{filter | type: :case_insensitive_text, value: String.downcase(value)}\n      result -> result\n    end\n  end\n\n  # Override to customize query building\n  @impl true\n  def build_query(query, field, %{value: value} = filter_value) do\n    field_atom = String.to_atom(field)\n    # Use ilike for case-insensitive matching\n    Ash.Query.filter(query, ilike(^ref(field_atom), ^\"%#{value}%\"))\n  end\nend\n```","title":"Example: Creating a Custom Text Filter - Custom Filters","ref":"custom-filters.html#example-creating-a-custom-text-filter"},{"type":"extras","doc":"- `--template` or `-t` - Choose the base filter to copy from\n- `--no-tests` - Skip generating test file\n- `--no-config` - Skip automatic configuration registration\n- `--no-setup` - Skip setup instructions","title":"Options - Custom Filters","ref":"custom-filters.html#options"},{"type":"extras","doc":"Here's a complete example of a custom slider filter:\n\n```elixir\ndefmodule MyApp.Filters.Slider do\n  @moduledoc \"\"\"\n  A range slider filter for numeric values.\n\n  Provides a visual slider interface for filtering numeric columns\n  with configurable min, max, and step values.\n  \"\"\"\n\n  use Cinder.Filter\n\n  @impl true\n  def render(column, current_value, theme, assigns) do\n    filter_options = Map.get(column, :filter_options, [])\n\n    min_value = get_option(filter_options, :min, 0)\n    max_value = get_option(filter_options, :max, 100)\n    step_value = get_option(filter_options, :step, 1)\n    current = current_value || min_value\n\n    assigns = %{\n      column: column,\n      current_value: current,\n      min_value: min_value,\n      max_value: max_value,\n      step_value: step_value,\n      theme: theme\n    }\n\n    ~H\"\"\"\n     \n       \n       \n         {@min_value} \n         {@current_value} \n         {@max_value} \n       \n     \n    \"\"\"\n  end\n\n  @impl true\n  def process(raw_value, column) when is_binary(raw_value) do\n    case Integer.parse(raw_value) do\n      {value, \"\"} ->\n        filter_options = Map.get(column, :filter_options, [])\n        operator = get_option(filter_options, :operator, :equals)\n\n        %{\n          type: :slider,\n          value: value,\n          operator: operator\n        }\n\n      _ -> nil\n    end\n  end\n\n  def process(_raw_value, _column), do: nil\n\n  @impl true\n  def validate(%{type: :slider, value: value, operator: operator})\n      when is_integer(value) and is_atom(operator) do\n    operator in [:equals, :greater_than, :less_than, :greater_than_or_equal, :less_than_or_equal]\n  end\n\n  def validate(_), do: false\n\n  @impl true\n  def default_options do\n    [\n      min: 0,\n      max: 100,\n      step: 1,\n      operator: :equals\n    ]\n  end\n\n  @impl true\n  def empty?(value) do\n    case value do\n      nil -> true\n      %{value: nil} -> true\n      _ -> false\n    end\n  end\n\n  @impl true\n  def build_query(query, field, filter_value) do\n    %{type: :slider, value: value, operator: operator} = filter_value\n\n    # Handle relationship fields using dot notation\n    if String.contains?(field, \".\") do\n      path_atoms = field |> String.split(\".\") |> Enum.map(&String.to_atom/1)\n      {rel_path, [field_atom]} = Enum.split(path_atoms, -1)\n\n      case operator do\n        :equals ->\n          Ash.Query.filter(query, exists(^rel_path, ^ref(field_atom) == ^value))\n        :greater_than ->\n          Ash.Query.filter(query, exists(^rel_path, ^ref(field_atom) > ^value))\n        :greater_than_or_equal ->\n          Ash.Query.filter(query, exists(^rel_path, ^ref(field_atom) >= ^value))\n        :less_than ->\n          Ash.Query.filter(query, exists(^rel_path, ^ref(field_atom) < ^value))\n        :less_than_or_equal ->\n          Ash.Query.filter(query, exists(^rel_path, ^ref(field_atom) <= ^value))\n        _ ->\n          query\n      end\n    else\n      # Direct field filtering\n      field_atom = String.to_atom(field)\n\n      case operator do\n        :equals ->\n          Ash.Query.filter(query, ^ref(field_atom) == ^value)\n        :greater_than ->\n          Ash.Query.filter(query, ^ref(field_atom) > ^value)\n        :greater_than_or_equal ->\n          Ash.Query.filter(query, ^ref(field_atom) >= ^value)\n        :less_than ->\n          Ash.Query.filter(query, ^ref(field_atom) < ^value)\n        :less_than_or_equal ->\n          Ash.Query.filter(query, ^ref(field_atom) <= ^value)\n        _ ->\n          query\n      end\n    end\n  end\nend\n```","title":"Quick Start - Custom Filters","ref":"custom-filters.html#quick-start"},{"type":"extras","doc":"","title":"Setup and Configuration - Custom Filters","ref":"custom-filters.html#setup-and-configuration"},{"type":"extras","doc":"Add your custom filters to your application configuration:\n\n```elixir\n# config/config.exs\nconfig :cinder, :filters, [\n  slider: MyApp.Filters.Slider,\n  color_picker: MyApp.Filters.ColorPicker,\n  date_picker: MyApp.Filters.DatePicker\n]\n```","title":"1. Configure Custom Filters - Custom Filters","ref":"custom-filters.html#1-configure-custom-filters"},{"type":"extras","doc":"Call `Cinder.setup()` in your application startup:\n\n```elixir\n# lib/my_app/application.ex\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    # Set up Cinder with configured filters\n    Cinder.setup()\n\n    children = [\n      # your supervisors...\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","title":"2. Initialize Cinder - Custom Filters","ref":"custom-filters.html#2-initialize-cinder"},{"type":"extras","doc":"Use your custom filter in column definitions:\n\n```elixir\n \n  <:col :let={product} field=\"price\" filter={:slider}\n        filter_options={[min: 0, max: 1000, step: 10]} sort>\n    ${product.price}\n  </:col>\n \n```","title":"3. Use in Tables - Custom Filters","ref":"custom-filters.html#3-use-in-tables"},{"type":"extras","doc":"All custom filters must implement the `Cinder.Filter` behaviour with these callbacks:","title":"The Cinder.Filter Behaviour - Custom Filters","ref":"custom-filters.html#the-cinder-filter-behaviour"},{"type":"extras","doc":"#### `render/4`\nRenders the filter UI component.\n\n```elixir\n@callback render(column :: map(), current_value :: any(), theme :: map(), assigns :: map()) :: Phoenix.LiveView.Rendered.t()\n```\n\n- `column`: Column configuration with `field`, `label`, `filter_options`\n- `current_value`: Current filter value (nil if no filter applied)\n- `theme`: Theme configuration with CSS classes\n- `assigns`: Additional assigns from the parent component\n\n#### `process/2`\nProcesses raw form/URL input into structured filter data.\n\n```elixir\n@callback process(raw_value :: any(), column :: map()) :: map() | nil\n```\n\nMust return a map with `:type`, `:value`, `:operator` keys, or `nil`.\n\n#### `validate/1`\nValidates a processed filter value.\n\n```elixir\n@callback validate(value :: any()) :: boolean()\n```\n\n#### `default_options/0`\nReturns default configuration options.\n\n```elixir\n@callback default_options() :: keyword()\n```\n\n#### `empty?/1`\nDetermines if a filter value is \"empty\" (no filtering applied).\n\n```elixir\n@callback empty?(value :: any()) :: boolean()\n```\n\n#### `build_query/3`\n**Critical for functionality!** Builds the Ash query for this filter.\n\n```elixir\n@callback build_query(query :: Ash.Query.t(), field :: String.t(), filter_value :: map()) :: Ash.Query.t()\n```","title":"Required Callbacks - Custom Filters","ref":"custom-filters.html#required-callbacks"},{"type":"extras","doc":"The `Cinder.Filter` module provides utilities via `use Cinder.Filter`:\n\n### `field_name/1`\nGenerates form field names:\n\n```elixir\n~H\"\"\"\n \n\"\"\"\n```\n\n### `get_option/3`\nSafely extracts options from filter configuration:\n\n```elixir\nfilter_options = Map.get(column, :filter_options, [])\nplaceholder = get_option(filter_options, :placeholder, \"Default text\")\n```","title":"Helper Functions - Custom Filters","ref":"custom-filters.html#helper-functions"},{"type":"extras","doc":"","title":"Common Patterns - Custom Filters","ref":"custom-filters.html#common-patterns"},{"type":"extras","doc":"```elixir\ndef process(raw_value, _column) when is_binary(raw_value) do\n  trimmed = String.trim(raw_value)\n\n  if trimmed == \"\" do\n    nil\n  else\n    %{\n      type: :my_filter,\n      value: trimmed,\n      operator: :equals\n    }\n  end\nend\n\ndef process(_raw_value, _column), do: nil\n```","title":"Simple Value Filters - Custom Filters","ref":"custom-filters.html#simple-value-filters"},{"type":"extras","doc":"```elixir\ndef process(%{\"min\" => min, \"max\" => max}, _column) do\n  with {min_val, \"\"} <- Integer.parse(min),\n       {max_val, \"\"} <- Integer.parse(max),\n       true <- min_val <= max_val do\n    %{\n      type: :my_range,\n      value: %{min: min_val, max: max_val},\n      operator: :between\n    }\n  else\n    _ -> nil\n  end\nend\n```","title":"Range Filters - Custom Filters","ref":"custom-filters.html#range-filters"},{"type":"extras","doc":"```elixir\ndef process(raw_values, _column) when is_list(raw_values) do\n  values = Enum.reject(raw_values, &(&1 == \"\" or is_nil(&1)))\n\n  if Enum.empty?(values) do\n    nil\n  else\n    %{\n      type: :my_multi,\n      value: values,\n      operator: :in\n    }\n  end\nend\n```","title":"Multi-Value Filters - Custom Filters","ref":"custom-filters.html#multi-value-filters"},{"type":"extras","doc":"","title":"Query Building Patterns - Custom Filters","ref":"custom-filters.html#query-building-patterns"},{"type":"extras","doc":"```elixir\ndef build_query(query, field, filter_value) do\n  %{value: value} = filter_value\n  field_atom = String.to_atom(field)\n  Ash.Query.filter(query, ^ref(field_atom) == ^value)\nend\n```","title":"Basic Field Filtering - Custom Filters","ref":"custom-filters.html#basic-field-filtering"},{"type":"extras","doc":"Handle dot notation fields like \"user.name\":\n\n```elixir\ndef build_query(query, field, filter_value) do\n  %{value: value} = filter_value\n\n  if String.contains?(field, \".\") do\n    # Handle relationship fields\n    path_atoms = field |> String.split(\".\") |> Enum.map(&String.to_atom/1)\n    {rel_path, [field_atom]} = Enum.split(path_atoms, -1)\n\n    Ash.Query.filter(query, exists(^rel_path, ^ref(field_atom) == ^value))\n  else\n    # Direct field filtering\n    field_atom = String.to_atom(field)\n    Ash.Query.filter(query, ^ref(field_atom) == ^value)\n  end\nend\n```","title":"Relationship Filtering - Custom Filters","ref":"custom-filters.html#relationship-filtering"},{"type":"extras","doc":"```elixir\ndef build_query(query, field, filter_value) do\n  %{type: :date_range, value: %{from: from_date, to: to_date}} = filter_value\n  field_atom = String.to_atom(field)\n\n  query\n  |> Ash.Query.filter(^ref(field_atom) >= ^from_date)\n  |> Ash.Query.filter(^ref(field_atom) <= ^to_date)\nend\n```","title":"Complex Queries - Custom Filters","ref":"custom-filters.html#complex-queries"},{"type":"extras","doc":"","title":"Testing Custom Filters - Custom Filters","ref":"custom-filters.html#testing-custom-filters"},{"type":"extras","doc":"```elixir\ndefmodule MyApp.Filters.SliderTest do\n  use ExUnit.Case\n  alias MyApp.Filters.Slider\n\n  describe \"process/2\" do\n    test \"processes valid integer values\" do\n      column = %{filter_options: [min: 0, max: 100]}\n\n      result = Slider.process(\"50\", column)\n\n      assert result == %{\n        type: :slider,\n        value: 50,\n        operator: :equals\n      }\n    end\n\n    test \"returns nil for invalid values\" do\n      result = Slider.process(\"invalid\", %{})\n      assert result == nil\n    end\n  end\n\n  describe \"validate/1\" do\n    test \"validates correct filter structure\" do\n      valid_filter = %{\n        type: :slider,\n        value: 75,\n        operator: :equals\n      }\n\n      assert Slider.validate(valid_filter) == true\n    end\n  end\n\n  describe \"build_query/3\" do\n    test \"builds correct query for direct field\" do\n      query = Ash.Query.new(MyApp.Product)\n      filter_value = %{type: :slider, value: 100, operator: :less_than_or_equal}\n\n      result = Slider.build_query(query, \"price\", filter_value)\n\n      # Test that the query has the correct filter\n      # Implementation depends on your testing setup\n    end\n  end\nend\n```","title":"Unit Tests - Custom Filters","ref":"custom-filters.html#unit-tests"},{"type":"extras","doc":"Check filter registration status:\n\n```elixir\n# List all registered filters\nCinder.Filters.Registry.list_filters()\n\n# Check if a filter is custom\nCinder.Filters.Registry.custom_filter?(:slider)\n\n# Get filter module\nCinder.Filters.Registry.get_filter(:slider)\n\n# Get default options\nCinder.Filters.Registry.default_options(:slider)\n\n# Check if registered\nCinder.Filters.Registry.registered?(:slider)\n```","title":"Registry Functions - Custom Filters","ref":"custom-filters.html#registry-functions"},{"type":"extras","doc":"","title":"Best Practices - Custom Filters","ref":"custom-filters.html#best-practices"},{"type":"extras","doc":"Always handle nil values, empty strings, and invalid input:\n\n```elixir\ndef process(raw_value, _column) when raw_value in [nil, \"\"], do: nil\ndef process(raw_value, column) when is_binary(raw_value) do\n  # Main processing logic\nend\ndef process(_raw_value, _column), do: nil\n```","title":"1. Handle Edge Cases - Custom Filters","ref":"custom-filters.html#1-handle-edge-cases"},{"type":"extras","doc":"```elixir\ndef default_options do\n  [\n    placeholder: \"Enter value...\",\n    case_sensitive: false,\n    operator: :contains\n  ]\nend\n```","title":"2. Provide Sensible Defaults - Custom Filters","ref":"custom-filters.html#2-provide-sensible-defaults"},{"type":"extras","doc":"```elixir\n# Column definition\n%{\n  field: \"score\",\n  filter: :slider,\n  filter_options: [\n    min: 0,\n    max: 1000,\n    step: 10,\n    operator: :less_than_or_equal\n  ]\n}\n```","title":"3. Make Filters Configurable - Custom Filters","ref":"custom-filters.html#3-make-filters-configurable"},{"type":"extras","doc":"```elixir\ndefmodule MyApp.Filters.Slider do\n  @moduledoc \"\"\"\n  Slider filter for numeric range filtering.","title":"4. Document Your Filters - Custom Filters","ref":"custom-filters.html#4-document-your-filters"},{"type":"extras","doc":"- `:min` - Minimum value (default: 0)\n  - `:max` - Maximum value (default: 100)\n  - `:step` - Step increment (default: 1)\n  - `:operator` - Comparison operator (default: :equals)","title":"Options - Custom Filters","ref":"custom-filters.html#options-1"},{"type":"extras","doc":"%{\n        field: \"price\",\n        filter: :slider,\n        filter_options: [min: 0, max: 1000, step: 50, operator: :less_than_or_equal]\n      }\n  \"\"\"\n\n  use Cinder.Filter\n  # ... implementation\nend\n```","title":"Usage - Custom Filters","ref":"custom-filters.html#usage"},{"type":"extras","doc":"This is the most critical callback - without it, your filter won't actually filter data!","title":"5. Implement build_query/3 Correctly - Custom Filters","ref":"custom-filters.html#5-implement-build_query-3-correctly"},{"type":"extras","doc":"","title":"Troubleshooting - Custom Filters","ref":"custom-filters.html#troubleshooting"},{"type":"extras","doc":"- Check configuration: `config :cinder, :filters, [slider: MyApp.Filters.Slider]`\n- Verify `Cinder.setup()` was called\n- Ensure column uses `filter={:slider}` (not `filter_type`)\n- Check console for registration errors","title":"Filter Not Appearing - Custom Filters","ref":"custom-filters.html#filter-not-appearing"},{"type":"extras","doc":"- Verify `build_query/3` is implemented\n- Check that `process/2` returns correct structure\n- Test with `Cinder.Filters.Registry.get_filter(:slider)`\n- Add logging to debug query building","title":"Filter Not Working - Custom Filters","ref":"custom-filters.html#filter-not-working"},{"type":"extras","doc":"- Check config syntax (proper map format)\n- Verify module names are correct\n- Run `Cinder.Filters.Registry.validate_custom_filters()`","title":"Configuration Issues - Custom Filters","ref":"custom-filters.html#configuration-issues"},{"type":"extras","doc":"- Ensure using `Cinder.Table.UrlSync`\n- Check `handle_params/3` implementation\n- Verify `url_state={@url_state}` attribute","title":"URL State Not Persisting - Custom Filters","ref":"custom-filters.html#url-state-not-persisting"},{"type":"extras","doc":"","title":"Advanced Topics - Custom Filters","ref":"custom-filters.html#advanced-topics"},{"type":"extras","doc":"```elixir\ndef render(column, current_value, theme, assigns) do\n  # Get options from database or context\n  options = get_dynamic_options(column.field, assigns)\n\n  assigns = Map.put(assigns, :dynamic_options, options)\n  # ... render with dynamic options\nend\n```","title":"Dynamic Filter Options - Custom Filters","ref":"custom-filters.html#dynamic-filter-options"},{"type":"extras","doc":"```elixir\ndef render(column, current_value, theme, assigns) do\n  # Access other filter values\n  category_filter = get_in(assigns, [:filters, \"category\"])\n\n  # Adjust options based on other filters\n  options = get_options_for_category(category_filter)\n  # ... render\nend\n```","title":"Filter Dependencies - Custom Filters","ref":"custom-filters.html#filter-dependencies"},{"type":"extras","doc":"```elixir\ndef validate(value) do\n  case value do\n    %{type: :my_filter, value: val} ->\n      # Custom business logic validation\n      val > 0 and val < 1000 and is_valid_for_business_rules(val)\n    _ ->\n      false\n  end\nend\n```","title":"Custom Validation - Custom Filters","ref":"custom-filters.html#custom-validation"},{"type":"extras","doc":"- Use `use Cinder.Filter` to get all necessary imports and behavior\n- Configure filters in `config.exs` with `:filters` key\n- Call `Cinder.setup()` once in application startup\n- Use `filter={:my_filter}` in column definitions\n- Always implement `build_query/3` - this is what actually filters data\n- Handle both direct fields and relationship fields (dot notation)\n- Provide fallback CSS classes for theme compatibility\n- Test all callbacks thoroughly, especially query building","title":"Key Points - Custom Filters","ref":"custom-filters.html#key-points"},{"type":"extras","doc":"# Changelog","title":"Changelog","ref":"changelog.html"},{"type":"extras","doc":"","title":"v0.3.0 (2025-06-23) - Changelog","ref":"changelog.html#v0-3-0-2025-06-23"},{"type":"extras","doc":"* Add `row_click` option for `Cinder.Table.table`, to make entire rows clickable\n* Support `scope` and `tenant` options to `Cinder.Table.table`\n  * `tenant` can also be passed in as part of the `query_opts` option\n* Support `timeout`, `authorize?`, and `max_concurrency` options in `query_opts`","title":"Features - Changelog","ref":"changelog.html#features"},{"type":"extras","doc":"* Tweaked layout of filters to avoid overlapping input content","title":"Bug fixes - Changelog","ref":"changelog.html#bug-fixes"},{"type":"extras","doc":"","title":"v0.2.1 (2025-06-19) - Changelog","ref":"changelog.html#v0-2-1-2025-06-19"},{"type":"extras","doc":"* Default to `date_range` fields for all datetime-related types","title":"Features - Changelog","ref":"changelog.html#features-1"},{"type":"extras","doc":"* Prevent crashing when an error occurs while loading table data - the error will be properly logged instead\n* Fix errors when attempting to filter on `NaiveDatetime` attribute","title":"Bug fixes - Changelog","ref":"changelog.html#bug-fixes-1"},{"type":"extras","doc":"","title":"v0.2.0 (2025-06-18) - Changelog","ref":"changelog.html#v0-2-0-2025-06-18"},{"type":"extras","doc":"* Allow a default theme to be specified for all tables, in application config (eg. `config :cinder, default_theme: \"dark\"`)\n* Reorder arguments to `UrlSync.handle_params` to be consistent with LiveView's `handle_params`\n  * Replace `Cinder.Table.UrlSync.handle_params(socket, params, url)` with `Cinder.Table.UrlSync.handle_params(params, uri, socket)`","title":"Features - Changelog","ref":"changelog.html#features-2"},{"type":"extras","doc":"","title":"v0.1.1 (2025-06-16) - Changelog","ref":"changelog.html#v0-1-1-2025-06-16"},{"type":"extras","doc":"* Fix bug where invalid sorts would sometimes raise `(Protocol.UndefinedError) protocol String.Chars not implemented for type Ash.Query (a struct)`\n* Fix incorrect environment specification for `sourcerer` and `igniter` dependencies - these should only ever be installed in `dev` and `test`\n* Fix styling of table row borders in `flowbite` theme (light mode)","title":"Bug fixes - Changelog","ref":"changelog.html#bug-fixes-2"},{"type":"extras","doc":"* Initial release","title":"v0.1.0 (2025-06-15) - Changelog","ref":"changelog.html#v0-1-0-2025-06-15"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}